# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.resources.types import user_list
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.services",
    marshal="google.ads.googleads.v8",
    manifest={
        "GetUserListRequest",
        "MutateUserListsRequest",
        "UserListOperation",
        "MutateUserListsResponse",
        "MutateUserListResult",
    },
)


class GetUserListRequest(proto.Message):
    r"""Request message for
    [UserListService.GetUserList][google.ads.googleads.v8.services.UserListService.GetUserList].

    Attributes:
        resource_name (str):
            Required. The resource name of the user list
            to fetch.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class MutateUserListsRequest(proto.Message):
    r"""Request message for
    [UserListService.MutateUserLists][google.ads.googleads.v8.services.UserListService.MutateUserLists].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose user
            lists are being modified.
        operations (Sequence[google.ads.googleads.v8.services.types.UserListOperation]):
            Required. The list of operations to perform
            on individual user lists.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=2, message="UserListOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)
    validate_only = proto.Field(proto.BOOL, number=4,)


class UserListOperation(proto.Message):
    r"""A single operation (create, update) on a user list.
    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v8.resources.types.UserList):
            Create operation: No resource name is
            expected for the new user list.
        update (google.ads.googleads.v8.resources.types.UserList):
            Update operation: The user list is expected
            to have a valid resource name.
        remove (str):
            Remove operation: A resource name for the removed user list
            is expected, in this format:

            ``customers/{customer_id}/userLists/{user_list_id}``
    """

    update_mask = proto.Field(
        proto.MESSAGE, number=4, message=field_mask_pb2.FieldMask,
    )
    create = proto.Field(
        proto.MESSAGE, number=1, oneof="operation", message=user_list.UserList,
    )
    update = proto.Field(
        proto.MESSAGE, number=2, oneof="operation", message=user_list.UserList,
    )
    remove = proto.Field(proto.STRING, number=3, oneof="operation",)


class MutateUserListsResponse(proto.Message):
    r"""Response message for user list mutate.
    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (e.g. auth errors), we return an RPC
            level error.
        results (Sequence[google.ads.googleads.v8.services.types.MutateUserListResult]):
            All results for the mutate.
    """

    partial_failure_error = proto.Field(
        proto.MESSAGE, number=3, message=status_pb2.Status,
    )
    results = proto.RepeatedField(
        proto.MESSAGE, number=2, message="MutateUserListResult",
    )


class MutateUserListResult(proto.Message):
    r"""The result for the user list mutate.
    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.resources.types import customer_client_link
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.services",
    marshal="google.ads.googleads.v8",
    manifest={
        "GetCustomerClientLinkRequest",
        "MutateCustomerClientLinkRequest",
        "CustomerClientLinkOperation",
        "MutateCustomerClientLinkResponse",
        "MutateCustomerClientLinkResult",
    },
)


class GetCustomerClientLinkRequest(proto.Message):
    r"""Request message for
    [CustomerClientLinkService.GetCustomerClientLink][google.ads.googleads.v8.services.CustomerClientLinkService.GetCustomerClientLink].

    Attributes:
        resource_name (str):
            Required. The resource name of the customer
            client link to fetch.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class MutateCustomerClientLinkRequest(proto.Message):
    r"""Request message for
    [CustomerClientLinkService.MutateCustomerClientLink][google.ads.googleads.v8.services.CustomerClientLinkService.MutateCustomerClientLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            customer link are being modified.
        operation (google.ads.googleads.v8.services.types.CustomerClientLinkOperation):
            Required. The operation to perform on the
            individual CustomerClientLink.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operation = proto.Field(
        proto.MESSAGE, number=2, message="CustomerClientLinkOperation",
    )
    validate_only = proto.Field(proto.BOOL, number=3,)


class CustomerClientLinkOperation(proto.Message):
    r"""A single operation (create, update) on a CustomerClientLink.
    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v8.resources.types.CustomerClientLink):
            Create operation: No resource name is
            expected for the new link.
        update (google.ads.googleads.v8.resources.types.CustomerClientLink):
            Update operation: The link is expected to
            have a valid resource name.
    """

    update_mask = proto.Field(
        proto.MESSAGE, number=4, message=field_mask_pb2.FieldMask,
    )
    create = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=customer_client_link.CustomerClientLink,
    )
    update = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=customer_client_link.CustomerClientLink,
    )


class MutateCustomerClientLinkResponse(proto.Message):
    r"""Response message for a CustomerClientLink mutate.
    Attributes:
        result (google.ads.googleads.v8.services.types.MutateCustomerClientLinkResult):
            A result that identifies the resource
            affected by the mutate request.
    """

    result = proto.Field(
        proto.MESSAGE, number=1, message="MutateCustomerClientLinkResult",
    )


class MutateCustomerClientLinkResult(proto.Message):
    r"""The result for a single customer client link mutate.
    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v10.common.types import asset_usage


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.resources",
    marshal="google.ads.googleads.v10",
    manifest={"AdGroupAdAssetCombinationView",},
)


class AdGroupAdAssetCombinationView(proto.Message):
    r"""A view on the usage of ad group ad asset combination.
    Now we only support AdGroupAdAssetCombinationView for Responsive
    Search Ads, with more ad types planned for the future.

    Attributes:
        resource_name (str):
            Output only. The resource name of the ad group ad asset
            combination view. The combination ID is 128 bits long, where
            the upper 64 bits are stored in asset_combination_id_high,
            and the lower 64 bits are stored in
            asset_combination_id_low. AdGroupAd Asset Combination view
            resource names have the form:
            ``customers/{customer_id}/adGroupAdAssetCombinationViews/{AdGroupAd.ad_group_id}~{AdGroupAd.ad.ad_id}~{AssetCombination.asset_combination_id_low}~{AssetCombination.asset_combination_id_high}``
        served_assets (Sequence[google.ads.googleads.v10.common.types.AssetUsage]):
            Output only. Served assets.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    served_assets = proto.RepeatedField(
        proto.MESSAGE, number=2, message=asset_usage.AssetUsage,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.common.types import custom_parameter
from google.ads.googleads.v8.common.types import feed_common
from google.ads.googleads.v8.enums.types import app_store as gage_app_store
from google.ads.googleads.v8.enums.types import (
    call_conversion_reporting_state as gage_call_conversion_reporting_state,
)
from google.ads.googleads.v8.enums.types import price_extension_price_qualifier
from google.ads.googleads.v8.enums.types import price_extension_price_unit
from google.ads.googleads.v8.enums.types import price_extension_type
from google.ads.googleads.v8.enums.types import (
    promotion_extension_discount_modifier,
)
from google.ads.googleads.v8.enums.types import promotion_extension_occasion


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.common",
    marshal="google.ads.googleads.v8",
    manifest={
        "AppFeedItem",
        "CallFeedItem",
        "CalloutFeedItem",
        "LocationFeedItem",
        "AffiliateLocationFeedItem",
        "TextMessageFeedItem",
        "PriceFeedItem",
        "PriceOffer",
        "PromotionFeedItem",
        "StructuredSnippetFeedItem",
        "SitelinkFeedItem",
        "HotelCalloutFeedItem",
        "ImageFeedItem",
    },
)


class AppFeedItem(proto.Message):
    r"""Represents an App extension.
    Attributes:
        link_text (str):
            The visible text displayed when the link is
            rendered in an ad. This string must not be
            empty, and the length of this string should be
            between 1 and 25, inclusive.
        app_id (str):
            The store-specific ID for the target
            application. This string must not be empty.
        app_store (google.ads.googleads.v8.enums.types.AppStoreEnum.AppStore):
            The application store that the target
            application belongs to. This field is required.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects. This list must not be empty.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.
            Default value is "{lpurl}".
        url_custom_parameters (Sequence[google.ads.googleads.v8.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.
    """

    link_text = proto.Field(proto.STRING, number=9, optional=True,)
    app_id = proto.Field(proto.STRING, number=10, optional=True,)
    app_store = proto.Field(
        proto.ENUM, number=3, enum=gage_app_store.AppStoreEnum.AppStore,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=11,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=12,)
    tracking_url_template = proto.Field(proto.STRING, number=13, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=7, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=14, optional=True,)


class CallFeedItem(proto.Message):
    r"""Represents a Call extension.
    Attributes:
        phone_number (str):
            The advertiser's phone number to append to
            the ad. This string must not be empty.
        country_code (str):
            Uppercase two-letter country code of the
            advertiser's phone number. This string must not
            be empty.
        call_tracking_enabled (bool):
            Indicates whether call tracking is enabled.
            By default, call tracking is not enabled.
        call_conversion_action (str):
            The conversion action to attribute a call conversion to. If
            not set a default conversion action is used. This field only
            has effect if call_tracking_enabled is set to true.
            Otherwise this field is ignored.
        call_conversion_tracking_disabled (bool):
            If true, disable call conversion tracking.
            call_conversion_action should not be set if this is true.
            Optional.
        call_conversion_reporting_state (google.ads.googleads.v8.enums.types.CallConversionReportingStateEnum.CallConversionReportingState):
            Enum value that indicates whether this call
            extension uses its own call conversion setting
            (or just have call conversion disabled), or
            following the account level setting.
    """

    phone_number = proto.Field(proto.STRING, number=7, optional=True,)
    country_code = proto.Field(proto.STRING, number=8, optional=True,)
    call_tracking_enabled = proto.Field(proto.BOOL, number=9, optional=True,)
    call_conversion_action = proto.Field(
        proto.STRING, number=10, optional=True,
    )
    call_conversion_tracking_disabled = proto.Field(
        proto.BOOL, number=11, optional=True,
    )
    call_conversion_reporting_state = proto.Field(
        proto.ENUM,
        number=6,
        enum=gage_call_conversion_reporting_state.CallConversionReportingStateEnum.CallConversionReportingState,
    )


class CalloutFeedItem(proto.Message):
    r"""Represents a callout extension.
    Attributes:
        callout_text (str):
            The callout text.
            The length of this string should be between 1
            and 25, inclusive.
    """

    callout_text = proto.Field(proto.STRING, number=2, optional=True,)


class LocationFeedItem(proto.Message):
    r"""Represents a location extension.
    Attributes:
        business_name (str):
            The name of the business.
        address_line_1 (str):
            Line 1 of the business address.
        address_line_2 (str):
            Line 2 of the business address.
        city (str):
            City of the business address.
        province (str):
            Province of the business address.
        postal_code (str):
            Postal code of the business address.
        country_code (str):
            Country code of the business address.
        phone_number (str):
            Phone number of the business.
    """

    business_name = proto.Field(proto.STRING, number=9, optional=True,)
    address_line_1 = proto.Field(proto.STRING, number=10, optional=True,)
    address_line_2 = proto.Field(proto.STRING, number=11, optional=True,)
    city = proto.Field(proto.STRING, number=12, optional=True,)
    province = proto.Field(proto.STRING, number=13, optional=True,)
    postal_code = proto.Field(proto.STRING, number=14, optional=True,)
    country_code = proto.Field(proto.STRING, number=15, optional=True,)
    phone_number = proto.Field(proto.STRING, number=16, optional=True,)


class AffiliateLocationFeedItem(proto.Message):
    r"""Represents an affiliate location extension.
    Attributes:
        business_name (str):
            The name of the business.
        address_line_1 (str):
            Line 1 of the business address.
        address_line_2 (str):
            Line 2 of the business address.
        city (str):
            City of the business address.
        province (str):
            Province of the business address.
        postal_code (str):
            Postal code of the business address.
        country_code (str):
            Country code of the business address.
        phone_number (str):
            Phone number of the business.
        chain_id (int):
            Id of the retail chain that is advertised as
            a seller of your product.
        chain_name (str):
            Name of chain.
    """

    business_name = proto.Field(proto.STRING, number=11, optional=True,)
    address_line_1 = proto.Field(proto.STRING, number=12, optional=True,)
    address_line_2 = proto.Field(proto.STRING, number=13, optional=True,)
    city = proto.Field(proto.STRING, number=14, optional=True,)
    province = proto.Field(proto.STRING, number=15, optional=True,)
    postal_code = proto.Field(proto.STRING, number=16, optional=True,)
    country_code = proto.Field(proto.STRING, number=17, optional=True,)
    phone_number = proto.Field(proto.STRING, number=18, optional=True,)
    chain_id = proto.Field(proto.INT64, number=19, optional=True,)
    chain_name = proto.Field(proto.STRING, number=20, optional=True,)


class TextMessageFeedItem(proto.Message):
    r"""An extension that users can click on to send a text message
    to the advertiser.

    Attributes:
        business_name (str):
            The business name to prepend to the message
            text. This field is required.
        country_code (str):
            Uppercase two-letter country code of the
            advertiser's phone number. This field is
            required.
        phone_number (str):
            The advertiser's phone number the message
            will be sent to. Required.
        text (str):
            The text to show in the ad.
            This field is required.
        extension_text (str):
            The message extension_text populated in the messaging app.
    """

    business_name = proto.Field(proto.STRING, number=6, optional=True,)
    country_code = proto.Field(proto.STRING, number=7, optional=True,)
    phone_number = proto.Field(proto.STRING, number=8, optional=True,)
    text = proto.Field(proto.STRING, number=9, optional=True,)
    extension_text = proto.Field(proto.STRING, number=10, optional=True,)


class PriceFeedItem(proto.Message):
    r"""Represents a Price extension.
    Attributes:
        type_ (google.ads.googleads.v8.enums.types.PriceExtensionTypeEnum.PriceExtensionType):
            Price extension type of this extension.
        price_qualifier (google.ads.googleads.v8.enums.types.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier):
            Price qualifier for all offers of this price
            extension.
        tracking_url_template (str):
            Tracking URL template for all offers of this
            price extension.
        language_code (str):
            The code of the language used for this price
            extension.
        price_offerings (Sequence[google.ads.googleads.v8.common.types.PriceOffer]):
            The price offerings in this price extension.
        final_url_suffix (str):
            Tracking URL template for all offers of this
            price extension.
    """

    type_ = proto.Field(
        proto.ENUM,
        number=1,
        enum=price_extension_type.PriceExtensionTypeEnum.PriceExtensionType,
    )
    price_qualifier = proto.Field(
        proto.ENUM,
        number=2,
        enum=price_extension_price_qualifier.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier,
    )
    tracking_url_template = proto.Field(proto.STRING, number=7, optional=True,)
    language_code = proto.Field(proto.STRING, number=8, optional=True,)
    price_offerings = proto.RepeatedField(
        proto.MESSAGE, number=5, message="PriceOffer",
    )
    final_url_suffix = proto.Field(proto.STRING, number=9, optional=True,)


class PriceOffer(proto.Message):
    r"""Represents one price offer in a price extension.
    Attributes:
        header (str):
            Header text of this offer.
        description (str):
            Description text of this offer.
        price (google.ads.googleads.v8.common.types.Money):
            Price value of this offer.
        unit (google.ads.googleads.v8.enums.types.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit):
            Price unit for this offer.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
    """

    header = proto.Field(proto.STRING, number=7, optional=True,)
    description = proto.Field(proto.STRING, number=8, optional=True,)
    price = proto.Field(proto.MESSAGE, number=3, message=feed_common.Money,)
    unit = proto.Field(
        proto.ENUM,
        number=4,
        enum=price_extension_price_unit.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=9,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=10,)


class PromotionFeedItem(proto.Message):
    r"""Represents a Promotion extension.
    Attributes:
        promotion_target (str):
            A freeform description of what the promotion
            is targeting. This field is required.
        discount_modifier (google.ads.googleads.v8.enums.types.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier):
            Enum that modifies the qualification of the
            discount.
        promotion_start_date (str):
            Start date of when the promotion is eligible
            to be redeemed.
        promotion_end_date (str):
            Last date when the promotion is eligible to
            be redeemed.
        occasion (google.ads.googleads.v8.enums.types.PromotionExtensionOccasionEnum.PromotionExtensionOccasion):
            The occasion the promotion was intended for.
            If an occasion is set, the redemption window
            will need to fall within the date range
            associated with the occasion.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects. This field is required.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.
        url_custom_parameters (Sequence[google.ads.googleads.v8.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.
        language_code (str):
            The language of the promotion.
            Represented as BCP 47 language tag.
        percent_off (int):
            Percentage off discount in the promotion in micros. One
            million is equivalent to one percent. Either this or
            money_off_amount is required.
        money_amount_off (google.ads.googleads.v8.common.types.Money):
            Money amount off for discount in the promotion. Either this
            or percent_off is required.
        promotion_code (str):
            A code the user should use in order to be
            eligible for the promotion.
        orders_over_amount (google.ads.googleads.v8.common.types.Money):
            The amount the total order needs to be for
            the user to be eligible for the promotion.
    """

    promotion_target = proto.Field(proto.STRING, number=16, optional=True,)
    discount_modifier = proto.Field(
        proto.ENUM,
        number=2,
        enum=promotion_extension_discount_modifier.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier,
    )
    promotion_start_date = proto.Field(proto.STRING, number=19, optional=True,)
    promotion_end_date = proto.Field(proto.STRING, number=20, optional=True,)
    occasion = proto.Field(
        proto.ENUM,
        number=9,
        enum=promotion_extension_occasion.PromotionExtensionOccasionEnum.PromotionExtensionOccasion,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=21,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=22,)
    tracking_url_template = proto.Field(proto.STRING, number=23, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=13, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=24, optional=True,)
    language_code = proto.Field(proto.STRING, number=25, optional=True,)
    percent_off = proto.Field(proto.INT64, number=17, oneof="discount_type",)
    money_amount_off = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="discount_type",
        message=feed_common.Money,
    )
    promotion_code = proto.Field(
        proto.STRING, number=18, oneof="promotion_trigger",
    )
    orders_over_amount = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="promotion_trigger",
        message=feed_common.Money,
    )


class StructuredSnippetFeedItem(proto.Message):
    r"""Represents a structured snippet extension.
    Attributes:
        header (str):
            The header of the snippet.
            This string must not be empty.
        values (Sequence[str]):
            The values in the snippet.
            The maximum size of this collection is 10.
    """

    header = proto.Field(proto.STRING, number=3, optional=True,)
    values = proto.RepeatedField(proto.STRING, number=4,)


class SitelinkFeedItem(proto.Message):
    r"""Represents a sitelink extension.
    Attributes:
        link_text (str):
            URL display text for the sitelink.
            The length of this string should be between 1
            and 25, inclusive.
        line1 (str):
            First line of the description for the
            sitelink. If this value is set, line2 must also
            be set. The length of this string should be
            between 0 and 35, inclusive.
        line2 (str):
            Second line of the description for the
            sitelink. If this value is set, line1 must also
            be set. The length of this string should be
            between 0 and 35, inclusive.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.
        url_custom_parameters (Sequence[google.ads.googleads.v8.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            Final URL suffix to be appended to landing
            page URLs served with parallel tracking.
    """

    link_text = proto.Field(proto.STRING, number=9, optional=True,)
    line1 = proto.Field(proto.STRING, number=10, optional=True,)
    line2 = proto.Field(proto.STRING, number=11, optional=True,)
    final_urls = proto.RepeatedField(proto.STRING, number=12,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=13,)
    tracking_url_template = proto.Field(proto.STRING, number=14, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=7, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=15, optional=True,)


class HotelCalloutFeedItem(proto.Message):
    r"""Represents a hotel callout extension.
    Attributes:
        text (str):
            The callout text.
            The length of this string should be between 1
            and 25, inclusive.
        language_code (str):
            The language of the hotel callout text.
            IETF BCP 47 compliant language code.
    """

    text = proto.Field(proto.STRING, number=3, optional=True,)
    language_code = proto.Field(proto.STRING, number=4, optional=True,)


class ImageFeedItem(proto.Message):
    r"""Represents an advertiser provided image extension.
    Attributes:
        image_asset (str):
            Required. Resource name of the image asset.
    """

    image_asset = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.errors",
    marshal="google.ads.googleads.v8",
    manifest={"AuthorizationErrorEnum",},
)


class AuthorizationErrorEnum(proto.Message):
    r"""Container for enum describing possible authorization errors.    """

    class AuthorizationError(proto.Enum):
        r"""Enum describing possible authorization errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        USER_PERMISSION_DENIED = 2
        DEVELOPER_TOKEN_NOT_ON_ALLOWLIST = 13
        DEVELOPER_TOKEN_PROHIBITED = 4
        PROJECT_DISABLED = 5
        AUTHORIZATION_ERROR = 6
        ACTION_NOT_PERMITTED = 7
        INCOMPLETE_SIGNUP = 8
        CUSTOMER_NOT_ENABLED = 24
        MISSING_TOS = 9
        DEVELOPER_TOKEN_NOT_APPROVED = 10
        INVALID_LOGIN_CUSTOMER_ID_SERVING_CUSTOMER_ID_COMBINATION = 11
        SERVICE_ACCESS_DENIED = 12
        ACCESS_DENIED_FOR_ACCOUNT_TYPE = 25


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.common.types import asset_types
from google.ads.googleads.v8.common.types import custom_parameter
from google.ads.googleads.v8.common.types import policy
from google.ads.googleads.v8.enums.types import asset_type
from google.ads.googleads.v8.enums.types import policy_approval_status
from google.ads.googleads.v8.enums.types import policy_review_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.resources",
    marshal="google.ads.googleads.v8",
    manifest={"Asset", "AssetPolicySummary",},
)


class Asset(proto.Message):
    r"""Asset is a part of an ad which can be shared across multiple
    ads. It can be an image (ImageAsset), a video
    (YoutubeVideoAsset), etc. Assets are immutable and cannot be
    removed. To stop an asset from serving, remove the asset from
    the entity that is using it.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset. Asset resource
            names have the form:

            ``customers/{customer_id}/assets/{asset_id}``
        id (int):
            Output only. The ID of the asset.
        name (str):
            Optional name of the asset.
        type_ (google.ads.googleads.v8.enums.types.AssetTypeEnum.AssetType):
            Output only. Type of the asset.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.
        url_custom_parameters (Sequence[google.ads.googleads.v8.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.
        policy_summary (google.ads.googleads.v8.resources.types.AssetPolicySummary):
            Output only. Policy information for the
            asset.
        youtube_video_asset (google.ads.googleads.v8.common.types.YoutubeVideoAsset):
            Immutable. A YouTube video asset.
        media_bundle_asset (google.ads.googleads.v8.common.types.MediaBundleAsset):
            Immutable. A media bundle asset.
        image_asset (google.ads.googleads.v8.common.types.ImageAsset):
            Output only. An image asset.
        text_asset (google.ads.googleads.v8.common.types.TextAsset):
            Output only. A text asset.
        lead_form_asset (google.ads.googleads.v8.common.types.LeadFormAsset):
            A lead form asset.
        book_on_google_asset (google.ads.googleads.v8.common.types.BookOnGoogleAsset):
            A book on google asset.
        promotion_asset (google.ads.googleads.v8.common.types.PromotionAsset):
            A promotion asset.
        callout_asset (google.ads.googleads.v8.common.types.CalloutAsset):
            A callout asset.
        structured_snippet_asset (google.ads.googleads.v8.common.types.StructuredSnippetAsset):
            A structured snippet asset.
        sitelink_asset (google.ads.googleads.v8.common.types.SitelinkAsset):
            A sitelink asset.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.INT64, number=11, optional=True,)
    name = proto.Field(proto.STRING, number=12, optional=True,)
    type_ = proto.Field(
        proto.ENUM, number=4, enum=asset_type.AssetTypeEnum.AssetType,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=14,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=16,)
    tracking_url_template = proto.Field(proto.STRING, number=17, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=18, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=19, optional=True,)
    policy_summary = proto.Field(
        proto.MESSAGE, number=13, message="AssetPolicySummary",
    )
    youtube_video_asset = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="asset_data",
        message=asset_types.YoutubeVideoAsset,
    )
    media_bundle_asset = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="asset_data",
        message=asset_types.MediaBundleAsset,
    )
    image_asset = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="asset_data",
        message=asset_types.ImageAsset,
    )
    text_asset = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="asset_data",
        message=asset_types.TextAsset,
    )
    lead_form_asset = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="asset_data",
        message=asset_types.LeadFormAsset,
    )
    book_on_google_asset = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="asset_data",
        message=asset_types.BookOnGoogleAsset,
    )
    promotion_asset = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="asset_data",
        message=asset_types.PromotionAsset,
    )
    callout_asset = proto.Field(
        proto.MESSAGE,
        number=20,
        oneof="asset_data",
        message=asset_types.CalloutAsset,
    )
    structured_snippet_asset = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="asset_data",
        message=asset_types.StructuredSnippetAsset,
    )
    sitelink_asset = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="asset_data",
        message=asset_types.SitelinkAsset,
    )


class AssetPolicySummary(proto.Message):
    r"""Contains policy information for an asset.
    Attributes:
        policy_topic_entries (Sequence[google.ads.googleads.v8.common.types.PolicyTopicEntry]):
            Output only. The list of policy findings for
            this asset.
        review_status (google.ads.googleads.v8.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Output only. Where in the review process this
            asset is.
        approval_status (google.ads.googleads.v8.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            Output only. The overall approval status of
            this asset, calculated based on the status of
            its individual policy topic entries.
    """

    policy_topic_entries = proto.RepeatedField(
        proto.MESSAGE, number=1, message=policy.PolicyTopicEntry,
    )
    review_status = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from typing import Callable, Dict, Optional, Sequence, Tuple

from google.api_core import grpc_helpers
from google.api_core import gapic_v1
import google.auth  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore

import grpc  # type: ignore

from google.ads.googleads.v9.services.types import (
    smart_campaign_suggest_service,
)
from .base import SmartCampaignSuggestServiceTransport, DEFAULT_CLIENT_INFO


class SmartCampaignSuggestServiceGrpcTransport(
    SmartCampaignSuggestServiceTransport
):
    """gRPC backend transport for SmartCampaignSuggestService.

    Service to get suggestions for Smart Campaigns.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    def __init__(
        self,
        *,
        host: str = "googleads.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Sequence[str] = None,
        channel: grpc.Channel = None,
        api_mtls_endpoint: str = None,
        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
        ssl_channel_credentials: grpc.ChannelCredentials = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            channel (Optional[grpc.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for grpc channel. It is ignored if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
              creation failed for any reason.
        """
        self._ssl_channel_credentials = ssl_channel_credentials

        if channel:
            # Sanity check: Ensure that channel and credentials are not both
            # provided.
            credentials = False

            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        elif api_mtls_endpoint:
            warnings.warn(
                "api_mtls_endpoint and client_cert_source are deprecated",
                DeprecationWarning,
            )

            host = (
                api_mtls_endpoint
                if ":" in api_mtls_endpoint
                else api_mtls_endpoint + ":443"
            )

            if credentials is None:
                credentials, _ = google.auth.default(
                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
                )

            # Create SSL credentials with client_cert_source or application
            # default SSL credentials.
            if client_cert_source:
                cert, key = client_cert_source()
                ssl_credentials = grpc.ssl_channel_credentials(
                    certificate_chain=cert, private_key=key
                )
            else:
                ssl_credentials = SslCredentials().ssl_credentials

            # create a new channel. The provided one is ignored.
            self._grpc_channel = type(self).create_channel(
                host,
                credentials=credentials,
                credentials_file=credentials_file,
                ssl_credentials=ssl_credentials,
                scopes=scopes or self.AUTH_SCOPES,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            self._ssl_channel_credentials = ssl_credentials
        else:
            host = host if ":" in host else host + ":443"

            if credentials is None:
                credentials, _ = google.auth.default(scopes=self.AUTH_SCOPES)

            # create a new channel. The provided one is ignored.
            self._grpc_channel = type(self).create_channel(
                host,
                credentials=credentials,
                ssl_credentials=ssl_channel_credentials,
                scopes=self.AUTH_SCOPES,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        self._stubs = {}  # type: Dict[str, Callable]

        # Run the base constructor.
        super().__init__(
            host=host, credentials=credentials, client_info=client_info,
        )

    @classmethod
    def create_channel(
        cls,
        host: str = "googleads.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        scopes: Optional[Sequence[str]] = None,
        **kwargs,
    ) -> grpc.Channel:
        """Create and return a gRPC channel object.
        Args:
            address (Optionsl[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return grpc_helpers.create_channel(
            host,
            credentials=credentials,
            scopes=scopes or cls.AUTH_SCOPES,
            **kwargs,
        )

    def close(self):
        self.grpc_channel.close()

    @property
    def grpc_channel(self) -> grpc.Channel:
        """Return the channel designed to connect to this service.
        """
        return self._grpc_channel

    @property
    def suggest_smart_campaign_budget_options(
        self,
    ) -> Callable[
        [
            smart_campaign_suggest_service.SuggestSmartCampaignBudgetOptionsRequest
        ],
        smart_campaign_suggest_service.SuggestSmartCampaignBudgetOptionsResponse,
    ]:
        r"""Return a callable for the suggest smart campaign budget
        options method over gRPC.

        Returns BudgetOption suggestions.

        Returns:
            Callable[[~.SuggestSmartCampaignBudgetOptionsRequest],
                    ~.SuggestSmartCampaignBudgetOptionsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "suggest_smart_campaign_budget_options" not in self._stubs:
            self._stubs[
                "suggest_smart_campaign_budget_options"
            ] = self.grpc_channel.unary_unary(
                "/google.ads.googleads.v9.services.SmartCampaignSuggestService/SuggestSmartCampaignBudgetOptions",
                request_serializer=smart_campaign_suggest_service.SuggestSmartCampaignBudgetOptionsRequest.serialize,
                response_deserializer=smart_campaign_suggest_service.SuggestSmartCampaignBudgetOptionsResponse.deserialize,
            )
        return self._stubs["suggest_smart_campaign_budget_options"]

    @property
    def suggest_smart_campaign_ad(
        self,
    ) -> Callable[
        [smart_campaign_suggest_service.SuggestSmartCampaignAdRequest],
        smart_campaign_suggest_service.SuggestSmartCampaignAdResponse,
    ]:
        r"""Return a callable for the suggest smart campaign ad method over gRPC.

        Suggests a Smart campaign ad compatible with the Ad
        family of resources, based on data points such as
        targeting and the business to advertise.

        Returns:
            Callable[[~.SuggestSmartCampaignAdRequest],
                    ~.SuggestSmartCampaignAdResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "suggest_smart_campaign_ad" not in self._stubs:
            self._stubs[
                "suggest_smart_campaign_ad"
            ] = self.grpc_channel.unary_unary(
                "/google.ads.googleads.v9.services.SmartCampaignSuggestService/SuggestSmartCampaignAd",
                request_serializer=smart_campaign_suggest_service.SuggestSmartCampaignAdRequest.serialize,
                response_deserializer=smart_campaign_suggest_service.SuggestSmartCampaignAdResponse.deserialize,
            )
        return self._stubs["suggest_smart_campaign_ad"]

    @property
    def suggest_keyword_themes(
        self,
    ) -> Callable[
        [smart_campaign_suggest_service.SuggestKeywordThemesRequest],
        smart_campaign_suggest_service.SuggestKeywordThemesResponse,
    ]:
        r"""Return a callable for the suggest keyword themes method over gRPC.

        Suggests keyword themes to advertise on.

        Returns:
            Callable[[~.SuggestKeywordThemesRequest],
                    ~.SuggestKeywordThemesResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "suggest_keyword_themes" not in self._stubs:
            self._stubs[
                "suggest_keyword_themes"
            ] = self.grpc_channel.unary_unary(
                "/google.ads.googleads.v9.services.SmartCampaignSuggestService/SuggestKeywordThemes",
                request_serializer=smart_campaign_suggest_service.SuggestKeywordThemesRequest.serialize,
                response_deserializer=smart_campaign_suggest_service.SuggestKeywordThemesResponse.deserialize,
            )
        return self._stubs["suggest_keyword_themes"]


__all__ = ("SmartCampaignSuggestServiceGrpcTransport",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.enums.types import ad_customizer_placeholder_field
from google.ads.googleads.v8.enums.types import (
    affiliate_location_placeholder_field,
)
from google.ads.googleads.v8.enums.types import app_placeholder_field
from google.ads.googleads.v8.enums.types import call_placeholder_field
from google.ads.googleads.v8.enums.types import callout_placeholder_field
from google.ads.googleads.v8.enums.types import custom_placeholder_field
from google.ads.googleads.v8.enums.types import dsa_page_feed_criterion_field
from google.ads.googleads.v8.enums.types import education_placeholder_field
from google.ads.googleads.v8.enums.types import feed_mapping_criterion_type
from google.ads.googleads.v8.enums.types import feed_mapping_status
from google.ads.googleads.v8.enums.types import flight_placeholder_field
from google.ads.googleads.v8.enums.types import hotel_placeholder_field
from google.ads.googleads.v8.enums.types import image_placeholder_field
from google.ads.googleads.v8.enums.types import job_placeholder_field
from google.ads.googleads.v8.enums.types import local_placeholder_field
from google.ads.googleads.v8.enums.types import (
    location_extension_targeting_criterion_field,
)
from google.ads.googleads.v8.enums.types import location_placeholder_field
from google.ads.googleads.v8.enums.types import message_placeholder_field
from google.ads.googleads.v8.enums.types import (
    placeholder_type as gage_placeholder_type,
)
from google.ads.googleads.v8.enums.types import price_placeholder_field
from google.ads.googleads.v8.enums.types import promotion_placeholder_field
from google.ads.googleads.v8.enums.types import real_estate_placeholder_field
from google.ads.googleads.v8.enums.types import sitelink_placeholder_field
from google.ads.googleads.v8.enums.types import (
    structured_snippet_placeholder_field,
)
from google.ads.googleads.v8.enums.types import travel_placeholder_field


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.resources",
    marshal="google.ads.googleads.v8",
    manifest={"FeedMapping", "AttributeFieldMapping",},
)


class FeedMapping(proto.Message):
    r"""A feed mapping.
    Attributes:
        resource_name (str):
            Immutable. The resource name of the feed mapping. Feed
            mapping resource names have the form:

            ``customers/{customer_id}/feedMappings/{feed_id}~{feed_mapping_id}``
        feed (str):
            Immutable. The feed of this feed mapping.
        attribute_field_mappings (Sequence[google.ads.googleads.v8.resources.types.AttributeFieldMapping]):
            Immutable. Feed attributes to field mappings.
            These mappings are a one-to-many relationship
            meaning that 1 feed attribute can be used to
            populate multiple placeholder fields, but 1
            placeholder field can only draw data from 1 feed
            attribute. Ad Customizer is an exception, 1
            placeholder field can be mapped to multiple feed
            attributes. Required.
        status (google.ads.googleads.v8.enums.types.FeedMappingStatusEnum.FeedMappingStatus):
            Output only. Status of the feed mapping.
            This field is read-only.
        placeholder_type (google.ads.googleads.v8.enums.types.PlaceholderTypeEnum.PlaceholderType):
            Immutable. The placeholder type of this
            mapping (i.e., if the mapping maps feed
            attributes to placeholder fields).
        criterion_type (google.ads.googleads.v8.enums.types.FeedMappingCriterionTypeEnum.FeedMappingCriterionType):
            Immutable. The criterion type of this mapping
            (i.e., if the mapping maps feed attributes to
            criterion fields).
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    feed = proto.Field(proto.STRING, number=7, optional=True,)
    attribute_field_mappings = proto.RepeatedField(
        proto.MESSAGE, number=5, message="AttributeFieldMapping",
    )
    status = proto.Field(
        proto.ENUM,
        number=6,
        enum=feed_mapping_status.FeedMappingStatusEnum.FeedMappingStatus,
    )
    placeholder_type = proto.Field(
        proto.ENUM,
        number=3,
        oneof="target",
        enum=gage_placeholder_type.PlaceholderTypeEnum.PlaceholderType,
    )
    criterion_type = proto.Field(
        proto.ENUM,
        number=4,
        oneof="target",
        enum=feed_mapping_criterion_type.FeedMappingCriterionTypeEnum.FeedMappingCriterionType,
    )


class AttributeFieldMapping(proto.Message):
    r"""Maps from feed attribute id to a placeholder or criterion
    field id.

    Attributes:
        feed_attribute_id (int):
            Immutable. Feed attribute from which to map.
        field_id (int):
            Output only. The placeholder field ID. If a
            placeholder field enum is not published in the
            current API version, then this field will be
            populated and the field oneof will be empty.
            This field is read-only.
        sitelink_field (google.ads.googleads.v8.enums.types.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField):
            Immutable. Sitelink Placeholder Fields.
        call_field (google.ads.googleads.v8.enums.types.CallPlaceholderFieldEnum.CallPlaceholderField):
            Immutable. Call Placeholder Fields.
        app_field (google.ads.googleads.v8.enums.types.AppPlaceholderFieldEnum.AppPlaceholderField):
            Immutable. App Placeholder Fields.
        location_field (google.ads.googleads.v8.enums.types.LocationPlaceholderFieldEnum.LocationPlaceholderField):
            Output only. Location Placeholder Fields.
            This field is read-only.
        affiliate_location_field (google.ads.googleads.v8.enums.types.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField):
            Output only. Affiliate Location Placeholder
            Fields. This field is read-only.
        callout_field (google.ads.googleads.v8.enums.types.CalloutPlaceholderFieldEnum.CalloutPlaceholderField):
            Immutable. Callout Placeholder Fields.
        structured_snippet_field (google.ads.googleads.v8.enums.types.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField):
            Immutable. Structured Snippet Placeholder
            Fields.
        message_field (google.ads.googleads.v8.enums.types.MessagePlaceholderFieldEnum.MessagePlaceholderField):
            Immutable. Message Placeholder Fields.
        price_field (google.ads.googleads.v8.enums.types.PricePlaceholderFieldEnum.PricePlaceholderField):
            Immutable. Price Placeholder Fields.
        promotion_field (google.ads.googleads.v8.enums.types.PromotionPlaceholderFieldEnum.PromotionPlaceholderField):
            Immutable. Promotion Placeholder Fields.
        ad_customizer_field (google.ads.googleads.v8.enums.types.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField):
            Immutable. Ad Customizer Placeholder Fields
        dsa_page_feed_field (google.ads.googleads.v8.enums.types.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField):
            Immutable. Dynamic Search Ad Page Feed
            Fields.
        location_extension_targeting_field (google.ads.googleads.v8.enums.types.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField):
            Immutable. Location Target Fields.
        education_field (google.ads.googleads.v8.enums.types.EducationPlaceholderFieldEnum.EducationPlaceholderField):
            Immutable. Education Placeholder Fields
        flight_field (google.ads.googleads.v8.enums.types.FlightPlaceholderFieldEnum.FlightPlaceholderField):
            Immutable. Flight Placeholder Fields
        custom_field (google.ads.googleads.v8.enums.types.CustomPlaceholderFieldEnum.CustomPlaceholderField):
            Immutable. Custom Placeholder Fields
        hotel_field (google.ads.googleads.v8.enums.types.HotelPlaceholderFieldEnum.HotelPlaceholderField):
            Immutable. Hotel Placeholder Fields
        real_estate_field (google.ads.googleads.v8.enums.types.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField):
            Immutable. Real Estate Placeholder Fields
        travel_field (google.ads.googleads.v8.enums.types.TravelPlaceholderFieldEnum.TravelPlaceholderField):
            Immutable. Travel Placeholder Fields
        local_field (google.ads.googleads.v8.enums.types.LocalPlaceholderFieldEnum.LocalPlaceholderField):
            Immutable. Local Placeholder Fields
        job_field (google.ads.googleads.v8.enums.types.JobPlaceholderFieldEnum.JobPlaceholderField):
            Immutable. Job Placeholder Fields
        image_field (google.ads.googleads.v8.enums.types.ImagePlaceholderFieldEnum.ImagePlaceholderField):
            Immutable. Image Placeholder Fields
    """

    feed_attribute_id = proto.Field(proto.INT64, number=24, optional=True,)
    field_id = proto.Field(proto.INT64, number=25, optional=True,)
    sitelink_field = proto.Field(
        proto.ENUM,
        number=3,
        oneof="field",
        enum=sitelink_placeholder_field.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField,
    )
    call_field = proto.Field(
        proto.ENUM,
        number=4,
        oneof="field",
        enum=call_placeholder_field.CallPlaceholderFieldEnum.CallPlaceholderField,
    )
    app_field = proto.Field(
        proto.ENUM,
        number=5,
        oneof="field",
        enum=app_placeholder_field.AppPlaceholderFieldEnum.AppPlaceholderField,
    )
    location_field = proto.Field(
        proto.ENUM,
        number=6,
        oneof="field",
        enum=location_placeholder_field.LocationPlaceholderFieldEnum.LocationPlaceholderField,
    )
    affiliate_location_field = proto.Field(
        proto.ENUM,
        number=7,
        oneof="field",
        enum=affiliate_location_placeholder_field.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField,
    )
    callout_field = proto.Field(
        proto.ENUM,
        number=8,
        oneof="field",
        enum=callout_placeholder_field.CalloutPlaceholderFieldEnum.CalloutPlaceholderField,
    )
    structured_snippet_field = proto.Field(
        proto.ENUM,
        number=9,
        oneof="field",
        enum=structured_snippet_placeholder_field.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField,
    )
    message_field = proto.Field(
        proto.ENUM,
        number=10,
        oneof="field",
        enum=message_placeholder_field.MessagePlaceholderFieldEnum.MessagePlaceholderField,
    )
    price_field = proto.Field(
        proto.ENUM,
        number=11,
        oneof="field",
        enum=price_placeholder_field.PricePlaceholderFieldEnum.PricePlaceholderField,
    )
    promotion_field = proto.Field(
        proto.ENUM,
        number=12,
        oneof="field",
        enum=promotion_placeholder_field.PromotionPlaceholderFieldEnum.PromotionPlaceholderField,
    )
    ad_customizer_field = proto.Field(
        proto.ENUM,
        number=13,
        oneof="field",
        enum=ad_customizer_placeholder_field.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField,
    )
    dsa_page_feed_field = proto.Field(
        proto.ENUM,
        number=14,
        oneof="field",
        enum=dsa_page_feed_criterion_field.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField,
    )
    location_extension_targeting_field = proto.Field(
        proto.ENUM,
        number=15,
        oneof="field",
        enum=location_extension_targeting_criterion_field.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField,
    )
    education_field = proto.Field(
        proto.ENUM,
        number=16,
        oneof="field",
        enum=education_placeholder_field.EducationPlaceholderFieldEnum.EducationPlaceholderField,
    )
    flight_field = proto.Field(
        proto.ENUM,
        number=17,
        oneof="field",
        enum=flight_placeholder_field.FlightPlaceholderFieldEnum.FlightPlaceholderField,
    )
    custom_field = proto.Field(
        proto.ENUM,
        number=18,
        oneof="field",
        enum=custom_placeholder_field.CustomPlaceholderFieldEnum.CustomPlaceholderField,
    )
    hotel_field = proto.Field(
        proto.ENUM,
        number=19,
        oneof="field",
        enum=hotel_placeholder_field.HotelPlaceholderFieldEnum.HotelPlaceholderField,
    )
    real_estate_field = proto.Field(
        proto.ENUM,
        number=20,
        oneof="field",
        enum=real_estate_placeholder_field.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField,
    )
    travel_field = proto.Field(
        proto.ENUM,
        number=21,
        oneof="field",
        enum=travel_placeholder_field.TravelPlaceholderFieldEnum.TravelPlaceholderField,
    )
    local_field = proto.Field(
        proto.ENUM,
        number=22,
        oneof="field",
        enum=local_placeholder_field.LocalPlaceholderFieldEnum.LocalPlaceholderField,
    )
    job_field = proto.Field(
        proto.ENUM,
        number=23,
        oneof="field",
        enum=job_placeholder_field.JobPlaceholderFieldEnum.JobPlaceholderField,
    )
    image_field = proto.Field(
        proto.ENUM,
        number=26,
        oneof="field",
        enum=image_placeholder_field.ImagePlaceholderFieldEnum.ImagePlaceholderField,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.common.types import offline_user_data


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.services",
    marshal="google.ads.googleads.v8",
    manifest={
        "UploadUserDataRequest",
        "UserDataOperation",
        "UploadUserDataResponse",
    },
)


class UploadUserDataRequest(proto.Message):
    r"""Request message for
    [UserDataService.UploadUserData][google.ads.googleads.v8.services.UserDataService.UploadUserData]

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which to
            update the user data.
        operations (Sequence[google.ads.googleads.v8.services.types.UserDataOperation]):
            Required. The list of operations to be done.
        customer_match_user_list_metadata (google.ads.googleads.v8.common.types.CustomerMatchUserListMetadata):
            Metadata for data updates to a Customer Match
            user list.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=3, message="UserDataOperation",
    )
    customer_match_user_list_metadata = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="metadata",
        message=offline_user_data.CustomerMatchUserListMetadata,
    )


class UserDataOperation(proto.Message):
    r"""Operation to be made for the UploadUserDataRequest.
    Attributes:
        create (google.ads.googleads.v8.common.types.UserData):
            The list of user data to be appended to the
            user list.
        remove (google.ads.googleads.v8.common.types.UserData):
            The list of user data to be removed from the
            user list.
    """

    create = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=offline_user_data.UserData,
    )
    remove = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=offline_user_data.UserData,
    )


class UploadUserDataResponse(proto.Message):
    r"""Response message for
    [UserDataService.UploadUserData][google.ads.googleads.v8.services.UserDataService.UploadUserData]

    Attributes:
        upload_date_time (str):
            The date time at which the request was received by API,
            formatted as "yyyy-mm-dd hh:mm:ss+|-hh:mm", e.g. "2019-01-01
            12:32:45-08:00".
        received_operations_count (int):
            Number of upload data operations received by
            API.
    """

    upload_date_time = proto.Field(proto.STRING, number=3, optional=True,)
    received_operations_count = proto.Field(
        proto.INT32, number=4, optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.common.types import policy
from google.ads.googleads.v8.common.types import value
from google.ads.googleads.v8.enums.types import resource_limit_type
from google.ads.googleads.v8.errors.types import (
    access_invitation_error as gage_access_invitation_error,
)
from google.ads.googleads.v8.errors.types import (
    account_budget_proposal_error as gage_account_budget_proposal_error,
)
from google.ads.googleads.v8.errors.types import (
    account_link_error as gage_account_link_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_customizer_error as gage_ad_customizer_error,
)
from google.ads.googleads.v8.errors.types import ad_error as gage_ad_error
from google.ads.googleads.v8.errors.types import (
    ad_group_ad_error as gage_ad_group_ad_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_group_bid_modifier_error as gage_ad_group_bid_modifier_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_group_criterion_error as gage_ad_group_criterion_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_group_error as gage_ad_group_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_group_feed_error as gage_ad_group_feed_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_parameter_error as gage_ad_parameter_error,
)
from google.ads.googleads.v8.errors.types import (
    ad_sharing_error as gage_ad_sharing_error,
)
from google.ads.googleads.v8.errors.types import adx_error as gage_adx_error
from google.ads.googleads.v8.errors.types import asset_error as gage_asset_error
from google.ads.googleads.v8.errors.types import (
    asset_link_error as gage_asset_link_error,
)
from google.ads.googleads.v8.errors.types import (
    authentication_error as gage_authentication_error,
)
from google.ads.googleads.v8.errors.types import (
    authorization_error as gage_authorization_error,
)
from google.ads.googleads.v8.errors.types import (
    batch_job_error as gage_batch_job_error,
)
from google.ads.googleads.v8.errors.types import (
    bidding_error as gage_bidding_error,
)
from google.ads.googleads.v8.errors.types import (
    bidding_strategy_error as gage_bidding_strategy_error,
)
from google.ads.googleads.v8.errors.types import (
    billing_setup_error as gage_billing_setup_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_budget_error as gage_campaign_budget_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_criterion_error as gage_campaign_criterion_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_draft_error as gage_campaign_draft_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_error as gage_campaign_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_experiment_error as gage_campaign_experiment_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_feed_error as gage_campaign_feed_error,
)
from google.ads.googleads.v8.errors.types import (
    campaign_shared_set_error as gage_campaign_shared_set_error,
)
from google.ads.googleads.v8.errors.types import (
    change_event_error as gage_change_event_error,
)
from google.ads.googleads.v8.errors.types import (
    change_status_error as gage_change_status_error,
)
from google.ads.googleads.v8.errors.types import (
    collection_size_error as gage_collection_size_error,
)
from google.ads.googleads.v8.errors.types import (
    context_error as gage_context_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_action_error as gage_conversion_action_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_adjustment_upload_error as gage_conversion_adjustment_upload_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_custom_variable_error as gage_conversion_custom_variable_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_upload_error as gage_conversion_upload_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_value_rule_error as gage_conversion_value_rule_error,
)
from google.ads.googleads.v8.errors.types import (
    conversion_value_rule_set_error as gage_conversion_value_rule_set_error,
)
from google.ads.googleads.v8.errors.types import (
    country_code_error as gage_country_code_error,
)
from google.ads.googleads.v8.errors.types import (
    criterion_error as gage_criterion_error,
)
from google.ads.googleads.v8.errors.types import (
    currency_code_error as gage_currency_code_error,
)
from google.ads.googleads.v8.errors.types import (
    custom_audience_error as gage_custom_audience_error,
)
from google.ads.googleads.v8.errors.types import (
    custom_interest_error as gage_custom_interest_error,
)
from google.ads.googleads.v8.errors.types import (
    customer_client_link_error as gage_customer_client_link_error,
)
from google.ads.googleads.v8.errors.types import (
    customer_error as gage_customer_error,
)
from google.ads.googleads.v8.errors.types import (
    customer_feed_error as gage_customer_feed_error,
)
from google.ads.googleads.v8.errors.types import (
    customer_manager_link_error as gage_customer_manager_link_error,
)
from google.ads.googleads.v8.errors.types import (
    customer_user_access_error as gage_customer_user_access_error,
)
from google.ads.googleads.v8.errors.types import (
    database_error as gage_database_error,
)
from google.ads.googleads.v8.errors.types import date_error as gage_date_error
from google.ads.googleads.v8.errors.types import (
    date_range_error as gage_date_range_error,
)
from google.ads.googleads.v8.errors.types import (
    distinct_error as gage_distinct_error,
)
from google.ads.googleads.v8.errors.types import enum_error as gage_enum_error
from google.ads.googleads.v8.errors.types import (
    extension_feed_item_error as gage_extension_feed_item_error,
)
from google.ads.googleads.v8.errors.types import (
    extension_setting_error as gage_extension_setting_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_attribute_reference_error as gage_feed_attribute_reference_error,
)
from google.ads.googleads.v8.errors.types import feed_error as gage_feed_error
from google.ads.googleads.v8.errors.types import (
    feed_item_error as gage_feed_item_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_item_set_error as gage_feed_item_set_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_item_set_link_error as gage_feed_item_set_link_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_item_target_error as gage_feed_item_target_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_item_validation_error as gage_feed_item_validation_error,
)
from google.ads.googleads.v8.errors.types import (
    feed_mapping_error as gage_feed_mapping_error,
)
from google.ads.googleads.v8.errors.types import field_error as gage_field_error
from google.ads.googleads.v8.errors.types import (
    field_mask_error as gage_field_mask_error,
)
from google.ads.googleads.v8.errors.types import (
    function_error as gage_function_error,
)
from google.ads.googleads.v8.errors.types import (
    function_parsing_error as gage_function_parsing_error,
)
from google.ads.googleads.v8.errors.types import (
    geo_target_constant_suggestion_error as gage_geo_target_constant_suggestion_error,
)
from google.ads.googleads.v8.errors.types import (
    header_error as gage_header_error,
)
from google.ads.googleads.v8.errors.types import id_error as gage_id_error
from google.ads.googleads.v8.errors.types import image_error as gage_image_error
from google.ads.googleads.v8.errors.types import (
    internal_error as gage_internal_error,
)
from google.ads.googleads.v8.errors.types import (
    invoice_error as gage_invoice_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_ad_group_error as gage_keyword_plan_ad_group_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_ad_group_keyword_error as gage_keyword_plan_ad_group_keyword_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_campaign_error as gage_keyword_plan_campaign_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_campaign_keyword_error as gage_keyword_plan_campaign_keyword_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_error as gage_keyword_plan_error,
)
from google.ads.googleads.v8.errors.types import (
    keyword_plan_idea_error as gage_keyword_plan_idea_error,
)
from google.ads.googleads.v8.errors.types import label_error as gage_label_error
from google.ads.googleads.v8.errors.types import (
    language_code_error as gage_language_code_error,
)
from google.ads.googleads.v8.errors.types import (
    list_operation_error as gage_list_operation_error,
)
from google.ads.googleads.v8.errors.types import (
    manager_link_error as gage_manager_link_error,
)
from google.ads.googleads.v8.errors.types import (
    media_bundle_error as gage_media_bundle_error,
)
from google.ads.googleads.v8.errors.types import (
    media_file_error as gage_media_file_error,
)
from google.ads.googleads.v8.errors.types import (
    media_upload_error as gage_media_upload_error,
)
from google.ads.googleads.v8.errors.types import (
    multiplier_error as gage_multiplier_error,
)
from google.ads.googleads.v8.errors.types import (
    mutate_error as gage_mutate_error,
)
from google.ads.googleads.v8.errors.types import (
    new_resource_creation_error as gage_new_resource_creation_error,
)
from google.ads.googleads.v8.errors.types import (
    not_allowlisted_error as gage_not_allowlisted_error,
)
from google.ads.googleads.v8.errors.types import (
    not_empty_error as gage_not_empty_error,
)
from google.ads.googleads.v8.errors.types import null_error as gage_null_error
from google.ads.googleads.v8.errors.types import (
    offline_user_data_job_error as gage_offline_user_data_job_error,
)
from google.ads.googleads.v8.errors.types import (
    operation_access_denied_error as gage_operation_access_denied_error,
)
from google.ads.googleads.v8.errors.types import (
    operator_error as gage_operator_error,
)
from google.ads.googleads.v8.errors.types import (
    partial_failure_error as gage_partial_failure_error,
)
from google.ads.googleads.v8.errors.types import (
    payments_account_error as gage_payments_account_error,
)
from google.ads.googleads.v8.errors.types import (
    policy_finding_error as gage_policy_finding_error,
)
from google.ads.googleads.v8.errors.types import (
    policy_validation_parameter_error as gage_policy_validation_parameter_error,
)
from google.ads.googleads.v8.errors.types import (
    policy_violation_error as gage_policy_violation_error,
)
from google.ads.googleads.v8.errors.types import query_error as gage_query_error
from google.ads.googleads.v8.errors.types import quota_error as gage_quota_error
from google.ads.googleads.v8.errors.types import range_error as gage_range_error
from google.ads.googleads.v8.errors.types import (
    reach_plan_error as gage_reach_plan_error,
)
from google.ads.googleads.v8.errors.types import (
    recommendation_error as gage_recommendation_error,
)
from google.ads.googleads.v8.errors.types import (
    region_code_error as gage_region_code_error,
)
from google.ads.googleads.v8.errors.types import (
    request_error as gage_request_error,
)
from google.ads.googleads.v8.errors.types import (
    resource_access_denied_error as gage_resource_access_denied_error,
)
from google.ads.googleads.v8.errors.types import (
    resource_count_limit_exceeded_error as gage_resource_count_limit_exceeded_error,
)
from google.ads.googleads.v8.errors.types import (
    setting_error as gage_setting_error,
)
from google.ads.googleads.v8.errors.types import (
    shared_criterion_error as gage_shared_criterion_error,
)
from google.ads.googleads.v8.errors.types import (
    shared_set_error as gage_shared_set_error,
)
from google.ads.googleads.v8.errors.types import (
    size_limit_error as gage_size_limit_error,
)
from google.ads.googleads.v8.errors.types import (
    string_format_error as gage_string_format_error,
)
from google.ads.googleads.v8.errors.types import (
    string_length_error as gage_string_length_error,
)
from google.ads.googleads.v8.errors.types import (
    third_party_app_analytics_link_error as gage_third_party_app_analytics_link_error,
)
from google.ads.googleads.v8.errors.types import (
    time_zone_error as gage_time_zone_error,
)
from google.ads.googleads.v8.errors.types import (
    url_field_error as gage_url_field_error,
)
from google.ads.googleads.v8.errors.types import (
    user_data_error as gage_user_data_error,
)
from google.ads.googleads.v8.errors.types import (
    user_list_error as gage_user_list_error,
)
from google.ads.googleads.v8.errors.types import (
    youtube_video_registration_error as gage_youtube_video_registration_error,
)
from google.protobuf import duration_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.errors",
    marshal="google.ads.googleads.v8",
    manifest={
        "GoogleAdsFailure",
        "GoogleAdsError",
        "ErrorCode",
        "ErrorLocation",
        "ErrorDetails",
        "PolicyViolationDetails",
        "PolicyFindingDetails",
        "QuotaErrorDetails",
        "ResourceCountDetails",
    },
)


class GoogleAdsFailure(proto.Message):
    r"""Describes how a GoogleAds API call failed. It's returned
    inside google.rpc.Status.details when a call fails.

    Attributes:
        errors (Sequence[google.ads.googleads.v8.errors.types.GoogleAdsError]):
            The list of errors that occurred.
        request_id (str):
            The unique ID of the request that is used for
            debugging purposes.
    """

    errors = proto.RepeatedField(
        proto.MESSAGE, number=1, message="GoogleAdsError",
    )
    request_id = proto.Field(proto.STRING, number=2,)


class GoogleAdsError(proto.Message):
    r"""GoogleAds-specific error.
    Attributes:
        error_code (google.ads.googleads.v8.errors.types.ErrorCode):
            An enum value that indicates which error
            occurred.
        message (str):
            A human-readable description of the error.
        trigger (google.ads.googleads.v8.common.types.Value):
            The value that triggered the error.
        location (google.ads.googleads.v8.errors.types.ErrorLocation):
            Describes the part of the request proto that
            caused the error.
        details (google.ads.googleads.v8.errors.types.ErrorDetails):
            Additional error details, which are returned
            by certain error codes. Most error codes do not
            include details.
    """

    error_code = proto.Field(proto.MESSAGE, number=1, message="ErrorCode",)
    message = proto.Field(proto.STRING, number=2,)
    trigger = proto.Field(proto.MESSAGE, number=3, message=value.Value,)
    location = proto.Field(proto.MESSAGE, number=4, message="ErrorLocation",)
    details = proto.Field(proto.MESSAGE, number=5, message="ErrorDetails",)


class ErrorCode(proto.Message):
    r"""The error reason represented by type and enum.
    Attributes:
        request_error (google.ads.googleads.v8.errors.types.RequestErrorEnum.RequestError):
            An error caused by the request
        bidding_strategy_error (google.ads.googleads.v8.errors.types.BiddingStrategyErrorEnum.BiddingStrategyError):
            An error with a Bidding Strategy mutate.
        url_field_error (google.ads.googleads.v8.errors.types.UrlFieldErrorEnum.UrlFieldError):
            An error with a URL field mutate.
        list_operation_error (google.ads.googleads.v8.errors.types.ListOperationErrorEnum.ListOperationError):
            An error with a list operation.
        query_error (google.ads.googleads.v8.errors.types.QueryErrorEnum.QueryError):
            An error with an AWQL query
        mutate_error (google.ads.googleads.v8.errors.types.MutateErrorEnum.MutateError):
            An error with a mutate
        field_mask_error (google.ads.googleads.v8.errors.types.FieldMaskErrorEnum.FieldMaskError):
            An error with a field mask
        authorization_error (google.ads.googleads.v8.errors.types.AuthorizationErrorEnum.AuthorizationError):
            An error encountered when trying to authorize
            a user.
        internal_error (google.ads.googleads.v8.errors.types.InternalErrorEnum.InternalError):
            An unexpected server-side error.
        quota_error (google.ads.googleads.v8.errors.types.QuotaErrorEnum.QuotaError):
            An error with the amonut of quota remaining.
        ad_error (google.ads.googleads.v8.errors.types.AdErrorEnum.AdError):
            An error with an Ad Group Ad mutate.
        ad_group_error (google.ads.googleads.v8.errors.types.AdGroupErrorEnum.AdGroupError):
            An error with an Ad Group mutate.
        campaign_budget_error (google.ads.googleads.v8.errors.types.CampaignBudgetErrorEnum.CampaignBudgetError):
            An error with a Campaign Budget mutate.
        campaign_error (google.ads.googleads.v8.errors.types.CampaignErrorEnum.CampaignError):
            An error with a Campaign mutate.
        authentication_error (google.ads.googleads.v8.errors.types.AuthenticationErrorEnum.AuthenticationError):
            Indicates failure to properly authenticate
            user.
        ad_group_criterion_error (google.ads.googleads.v8.errors.types.AdGroupCriterionErrorEnum.AdGroupCriterionError):
            Indicates failure to properly authenticate
            user.
        ad_customizer_error (google.ads.googleads.v8.errors.types.AdCustomizerErrorEnum.AdCustomizerError):
            The reasons for the ad customizer error
        ad_group_ad_error (google.ads.googleads.v8.errors.types.AdGroupAdErrorEnum.AdGroupAdError):
            The reasons for the ad group ad error
        ad_sharing_error (google.ads.googleads.v8.errors.types.AdSharingErrorEnum.AdSharingError):
            The reasons for the ad sharing error
        adx_error (google.ads.googleads.v8.errors.types.AdxErrorEnum.AdxError):
            The reasons for the adx error
        asset_error (google.ads.googleads.v8.errors.types.AssetErrorEnum.AssetError):
            The reasons for the asset error
        bidding_error (google.ads.googleads.v8.errors.types.BiddingErrorEnum.BiddingError):
            The reasons for the bidding errors
        campaign_criterion_error (google.ads.googleads.v8.errors.types.CampaignCriterionErrorEnum.CampaignCriterionError):
            The reasons for the campaign criterion error
        collection_size_error (google.ads.googleads.v8.errors.types.CollectionSizeErrorEnum.CollectionSizeError):
            The reasons for the collection size error
        country_code_error (google.ads.googleads.v8.errors.types.CountryCodeErrorEnum.CountryCodeError):
            The reasons for the country code error
        criterion_error (google.ads.googleads.v8.errors.types.CriterionErrorEnum.CriterionError):
            The reasons for the criterion error
        customer_error (google.ads.googleads.v8.errors.types.CustomerErrorEnum.CustomerError):
            The reasons for the customer error
        date_error (google.ads.googleads.v8.errors.types.DateErrorEnum.DateError):
            The reasons for the date error
        date_range_error (google.ads.googleads.v8.errors.types.DateRangeErrorEnum.DateRangeError):
            The reasons for the date range error
        distinct_error (google.ads.googleads.v8.errors.types.DistinctErrorEnum.DistinctError):
            The reasons for the distinct error
        feed_attribute_reference_error (google.ads.googleads.v8.errors.types.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError):
            The reasons for the feed attribute reference
            error
        function_error (google.ads.googleads.v8.errors.types.FunctionErrorEnum.FunctionError):
            The reasons for the function error
        function_parsing_error (google.ads.googleads.v8.errors.types.FunctionParsingErrorEnum.FunctionParsingError):
            The reasons for the function parsing error
        id_error (google.ads.googleads.v8.errors.types.IdErrorEnum.IdError):
            The reasons for the id error
        image_error (google.ads.googleads.v8.errors.types.ImageErrorEnum.ImageError):
            The reasons for the image error
        language_code_error (google.ads.googleads.v8.errors.types.LanguageCodeErrorEnum.LanguageCodeError):
            The reasons for the language code error
        media_bundle_error (google.ads.googleads.v8.errors.types.MediaBundleErrorEnum.MediaBundleError):
            The reasons for the media bundle error
        media_upload_error (google.ads.googleads.v8.errors.types.MediaUploadErrorEnum.MediaUploadError):
            The reasons for media uploading errors.
        media_file_error (google.ads.googleads.v8.errors.types.MediaFileErrorEnum.MediaFileError):
            The reasons for the media file error
        multiplier_error (google.ads.googleads.v8.errors.types.MultiplierErrorEnum.MultiplierError):
            The reasons for the multiplier error
        new_resource_creation_error (google.ads.googleads.v8.errors.types.NewResourceCreationErrorEnum.NewResourceCreationError):
            The reasons for the new resource creation
            error
        not_empty_error (google.ads.googleads.v8.errors.types.NotEmptyErrorEnum.NotEmptyError):
            The reasons for the not empty error
        null_error (google.ads.googleads.v8.errors.types.NullErrorEnum.NullError):
            The reasons for the null error
        operator_error (google.ads.googleads.v8.errors.types.OperatorErrorEnum.OperatorError):
            The reasons for the operator error
        range_error (google.ads.googleads.v8.errors.types.RangeErrorEnum.RangeError):
            The reasons for the range error
        recommendation_error (google.ads.googleads.v8.errors.types.RecommendationErrorEnum.RecommendationError):
            The reasons for error in applying a
            recommendation
        region_code_error (google.ads.googleads.v8.errors.types.RegionCodeErrorEnum.RegionCodeError):
            The reasons for the region code error
        setting_error (google.ads.googleads.v8.errors.types.SettingErrorEnum.SettingError):
            The reasons for the setting error
        string_format_error (google.ads.googleads.v8.errors.types.StringFormatErrorEnum.StringFormatError):
            The reasons for the string format error
        string_length_error (google.ads.googleads.v8.errors.types.StringLengthErrorEnum.StringLengthError):
            The reasons for the string length error
        operation_access_denied_error (google.ads.googleads.v8.errors.types.OperationAccessDeniedErrorEnum.OperationAccessDeniedError):
            The reasons for the operation access denied
            error
        resource_access_denied_error (google.ads.googleads.v8.errors.types.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError):
            The reasons for the resource access denied
            error
        resource_count_limit_exceeded_error (google.ads.googleads.v8.errors.types.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError):
            The reasons for the resource count limit
            exceeded error
        youtube_video_registration_error (google.ads.googleads.v8.errors.types.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError):
            The reasons for YouTube video registration
            errors.
        ad_group_bid_modifier_error (google.ads.googleads.v8.errors.types.AdGroupBidModifierErrorEnum.AdGroupBidModifierError):
            The reasons for the ad group bid modifier
            error
        context_error (google.ads.googleads.v8.errors.types.ContextErrorEnum.ContextError):
            The reasons for the context error
        field_error (google.ads.googleads.v8.errors.types.FieldErrorEnum.FieldError):
            The reasons for the field error
        shared_set_error (google.ads.googleads.v8.errors.types.SharedSetErrorEnum.SharedSetError):
            The reasons for the shared set error
        shared_criterion_error (google.ads.googleads.v8.errors.types.SharedCriterionErrorEnum.SharedCriterionError):
            The reasons for the shared criterion error
        campaign_shared_set_error (google.ads.googleads.v8.errors.types.CampaignSharedSetErrorEnum.CampaignSharedSetError):
            The reasons for the campaign shared set error
        conversion_action_error (google.ads.googleads.v8.errors.types.ConversionActionErrorEnum.ConversionActionError):
            The reasons for the conversion action error
        conversion_adjustment_upload_error (google.ads.googleads.v8.errors.types.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError):
            The reasons for the conversion adjustment
            upload error
        conversion_custom_variable_error (google.ads.googleads.v8.errors.types.ConversionCustomVariableErrorEnum.ConversionCustomVariableError):
            The reasons for the conversion custom
            variable error
        conversion_upload_error (google.ads.googleads.v8.errors.types.ConversionUploadErrorEnum.ConversionUploadError):
            The reasons for the conversion upload error
        conversion_value_rule_error (google.ads.googleads.v8.errors.types.ConversionValueRuleErrorEnum.ConversionValueRuleError):
            The reasons for the conversion value rule
            error
        conversion_value_rule_set_error (google.ads.googleads.v8.errors.types.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError):
            The reasons for the conversion value rule set
            error
        header_error (google.ads.googleads.v8.errors.types.HeaderErrorEnum.HeaderError):
            The reasons for the header error.
        database_error (google.ads.googleads.v8.errors.types.DatabaseErrorEnum.DatabaseError):
            The reasons for the database error.
        policy_finding_error (google.ads.googleads.v8.errors.types.PolicyFindingErrorEnum.PolicyFindingError):
            The reasons for the policy finding error.
        enum_error (google.ads.googleads.v8.errors.types.EnumErrorEnum.EnumError):
            The reason for enum error.
        keyword_plan_error (google.ads.googleads.v8.errors.types.KeywordPlanErrorEnum.KeywordPlanError):
            The reason for keyword plan error.
        keyword_plan_campaign_error (google.ads.googleads.v8.errors.types.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError):
            The reason for keyword plan campaign error.
        keyword_plan_campaign_keyword_error (google.ads.googleads.v8.errors.types.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError):
            The reason for keyword plan campaign keyword
            error.
        keyword_plan_ad_group_error (google.ads.googleads.v8.errors.types.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError):
            The reason for keyword plan ad group error.
        keyword_plan_ad_group_keyword_error (google.ads.googleads.v8.errors.types.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError):
            The reason for keyword plan ad group keyword
            error.
        keyword_plan_idea_error (google.ads.googleads.v8.errors.types.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError):
            The reason for keyword idea error.
        account_budget_proposal_error (google.ads.googleads.v8.errors.types.AccountBudgetProposalErrorEnum.AccountBudgetProposalError):
            The reasons for account budget proposal
            errors.
        user_list_error (google.ads.googleads.v8.errors.types.UserListErrorEnum.UserListError):
            The reasons for the user list error
        change_event_error (google.ads.googleads.v8.errors.types.ChangeEventErrorEnum.ChangeEventError):
            The reasons for the change event error
        change_status_error (google.ads.googleads.v8.errors.types.ChangeStatusErrorEnum.ChangeStatusError):
            The reasons for the change status error
        feed_error (google.ads.googleads.v8.errors.types.FeedErrorEnum.FeedError):
            The reasons for the feed error
        geo_target_constant_suggestion_error (google.ads.googleads.v8.errors.types.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError):
            The reasons for the geo target constant
            suggestion error.
        campaign_draft_error (google.ads.googleads.v8.errors.types.CampaignDraftErrorEnum.CampaignDraftError):
            The reasons for the campaign draft error
        feed_item_error (google.ads.googleads.v8.errors.types.FeedItemErrorEnum.FeedItemError):
            The reasons for the feed item error
        label_error (google.ads.googleads.v8.errors.types.LabelErrorEnum.LabelError):
            The reason for the label error.
        billing_setup_error (google.ads.googleads.v8.errors.types.BillingSetupErrorEnum.BillingSetupError):
            The reasons for the billing setup error
        customer_client_link_error (google.ads.googleads.v8.errors.types.CustomerClientLinkErrorEnum.CustomerClientLinkError):
            The reasons for the customer client link
            error
        customer_manager_link_error (google.ads.googleads.v8.errors.types.CustomerManagerLinkErrorEnum.CustomerManagerLinkError):
            The reasons for the customer manager link
            error
        feed_mapping_error (google.ads.googleads.v8.errors.types.FeedMappingErrorEnum.FeedMappingError):
            The reasons for the feed mapping error
        customer_feed_error (google.ads.googleads.v8.errors.types.CustomerFeedErrorEnum.CustomerFeedError):
            The reasons for the customer feed error
        ad_group_feed_error (google.ads.googleads.v8.errors.types.AdGroupFeedErrorEnum.AdGroupFeedError):
            The reasons for the ad group feed error
        campaign_feed_error (google.ads.googleads.v8.errors.types.CampaignFeedErrorEnum.CampaignFeedError):
            The reasons for the campaign feed error
        custom_interest_error (google.ads.googleads.v8.errors.types.CustomInterestErrorEnum.CustomInterestError):
            The reasons for the custom interest error
        campaign_experiment_error (google.ads.googleads.v8.errors.types.CampaignExperimentErrorEnum.CampaignExperimentError):
            The reasons for the campaign experiment error
        extension_feed_item_error (google.ads.googleads.v8.errors.types.ExtensionFeedItemErrorEnum.ExtensionFeedItemError):
            The reasons for the extension feed item error
        ad_parameter_error (google.ads.googleads.v8.errors.types.AdParameterErrorEnum.AdParameterError):
            The reasons for the ad parameter error
        feed_item_validation_error (google.ads.googleads.v8.errors.types.FeedItemValidationErrorEnum.FeedItemValidationError):
            The reasons for the feed item validation
            error
        extension_setting_error (google.ads.googleads.v8.errors.types.ExtensionSettingErrorEnum.ExtensionSettingError):
            The reasons for the extension setting error
        feed_item_set_error (google.ads.googleads.v8.errors.types.FeedItemSetErrorEnum.FeedItemSetError):
            The reasons for the feed item set error
        feed_item_set_link_error (google.ads.googleads.v8.errors.types.FeedItemSetLinkErrorEnum.FeedItemSetLinkError):
            The reasons for the feed item set link error
        feed_item_target_error (google.ads.googleads.v8.errors.types.FeedItemTargetErrorEnum.FeedItemTargetError):
            The reasons for the feed item target error
        policy_violation_error (google.ads.googleads.v8.errors.types.PolicyViolationErrorEnum.PolicyViolationError):
            The reasons for the policy violation error
        partial_failure_error (google.ads.googleads.v8.errors.types.PartialFailureErrorEnum.PartialFailureError):
            The reasons for the mutate job error
        policy_validation_parameter_error (google.ads.googleads.v8.errors.types.PolicyValidationParameterErrorEnum.PolicyValidationParameterError):
            The reasons for the policy validation
            parameter error
        size_limit_error (google.ads.googleads.v8.errors.types.SizeLimitErrorEnum.SizeLimitError):
            The reasons for the size limit error
        offline_user_data_job_error (google.ads.googleads.v8.errors.types.OfflineUserDataJobErrorEnum.OfflineUserDataJobError):
            The reasons for the offline user data job
            error.
        not_allowlisted_error (google.ads.googleads.v8.errors.types.NotAllowlistedErrorEnum.NotAllowlistedError):
            The reasons for the not allowlisted error
        manager_link_error (google.ads.googleads.v8.errors.types.ManagerLinkErrorEnum.ManagerLinkError):
            The reasons for the manager link error
        currency_code_error (google.ads.googleads.v8.errors.types.CurrencyCodeErrorEnum.CurrencyCodeError):
            The reasons for the currency code error
        access_invitation_error (google.ads.googleads.v8.errors.types.AccessInvitationErrorEnum.AccessInvitationError):
            The reasons for the access invitation error
        reach_plan_error (google.ads.googleads.v8.errors.types.ReachPlanErrorEnum.ReachPlanError):
            The reasons for the reach plan error
        invoice_error (google.ads.googleads.v8.errors.types.InvoiceErrorEnum.InvoiceError):
            The reasons for the invoice error
        payments_account_error (google.ads.googleads.v8.errors.types.PaymentsAccountErrorEnum.PaymentsAccountError):
            The reasons for errors in payments accounts
            service
        time_zone_error (google.ads.googleads.v8.errors.types.TimeZoneErrorEnum.TimeZoneError):
            The reasons for the time zone error
        asset_link_error (google.ads.googleads.v8.errors.types.AssetLinkErrorEnum.AssetLinkError):
            The reasons for the asset link error
        user_data_error (google.ads.googleads.v8.errors.types.UserDataErrorEnum.UserDataError):
            The reasons for the user data error.
        batch_job_error (google.ads.googleads.v8.errors.types.BatchJobErrorEnum.BatchJobError):
            The reasons for the batch job error
        account_link_error (google.ads.googleads.v8.errors.types.AccountLinkErrorEnum.AccountLinkError):
            The reasons for the account link status
            change error
        third_party_app_analytics_link_error (google.ads.googleads.v8.errors.types.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError):
            The reasons for the third party app analytics
            link mutate error
        customer_user_access_error (google.ads.googleads.v8.errors.types.CustomerUserAccessErrorEnum.CustomerUserAccessError):
            The reasons for the customer user access
            mutate error
        custom_audience_error (google.ads.googleads.v8.errors.types.CustomAudienceErrorEnum.CustomAudienceError):
            The reasons for the custom audience error
    """

    request_error = proto.Field(
        proto.ENUM,
        number=1,
        oneof="error_code",
        enum=gage_request_error.RequestErrorEnum.RequestError,
    )
    bidding_strategy_error = proto.Field(
        proto.ENUM,
        number=2,
        oneof="error_code",
        enum=gage_bidding_strategy_error.BiddingStrategyErrorEnum.BiddingStrategyError,
    )
    url_field_error = proto.Field(
        proto.ENUM,
        number=3,
        oneof="error_code",
        enum=gage_url_field_error.UrlFieldErrorEnum.UrlFieldError,
    )
    list_operation_error = proto.Field(
        proto.ENUM,
        number=4,
        oneof="error_code",
        enum=gage_list_operation_error.ListOperationErrorEnum.ListOperationError,
    )
    query_error = proto.Field(
        proto.ENUM,
        number=5,
        oneof="error_code",
        enum=gage_query_error.QueryErrorEnum.QueryError,
    )
    mutate_error = proto.Field(
        proto.ENUM,
        number=7,
        oneof="error_code",
        enum=gage_mutate_error.MutateErrorEnum.MutateError,
    )
    field_mask_error = proto.Field(
        proto.ENUM,
        number=8,
        oneof="error_code",
        enum=gage_field_mask_error.FieldMaskErrorEnum.FieldMaskError,
    )
    authorization_error = proto.Field(
        proto.ENUM,
        number=9,
        oneof="error_code",
        enum=gage_authorization_error.AuthorizationErrorEnum.AuthorizationError,
    )
    internal_error = proto.Field(
        proto.ENUM,
        number=10,
        oneof="error_code",
        enum=gage_internal_error.InternalErrorEnum.InternalError,
    )
    quota_error = proto.Field(
        proto.ENUM,
        number=11,
        oneof="error_code",
        enum=gage_quota_error.QuotaErrorEnum.QuotaError,
    )
    ad_error = proto.Field(
        proto.ENUM,
        number=12,
        oneof="error_code",
        enum=gage_ad_error.AdErrorEnum.AdError,
    )
    ad_group_error = proto.Field(
        proto.ENUM,
        number=13,
        oneof="error_code",
        enum=gage_ad_group_error.AdGroupErrorEnum.AdGroupError,
    )
    campaign_budget_error = proto.Field(
        proto.ENUM,
        number=14,
        oneof="error_code",
        enum=gage_campaign_budget_error.CampaignBudgetErrorEnum.CampaignBudgetError,
    )
    campaign_error = proto.Field(
        proto.ENUM,
        number=15,
        oneof="error_code",
        enum=gage_campaign_error.CampaignErrorEnum.CampaignError,
    )
    authentication_error = proto.Field(
        proto.ENUM,
        number=17,
        oneof="error_code",
        enum=gage_authentication_error.AuthenticationErrorEnum.AuthenticationError,
    )
    ad_group_criterion_error = proto.Field(
        proto.ENUM,
        number=18,
        oneof="error_code",
        enum=gage_ad_group_criterion_error.AdGroupCriterionErrorEnum.AdGroupCriterionError,
    )
    ad_customizer_error = proto.Field(
        proto.ENUM,
        number=19,
        oneof="error_code",
        enum=gage_ad_customizer_error.AdCustomizerErrorEnum.AdCustomizerError,
    )
    ad_group_ad_error = proto.Field(
        proto.ENUM,
        number=21,
        oneof="error_code",
        enum=gage_ad_group_ad_error.AdGroupAdErrorEnum.AdGroupAdError,
    )
    ad_sharing_error = proto.Field(
        proto.ENUM,
        number=24,
        oneof="error_code",
        enum=gage_ad_sharing_error.AdSharingErrorEnum.AdSharingError,
    )
    adx_error = proto.Field(
        proto.ENUM,
        number=25,
        oneof="error_code",
        enum=gage_adx_error.AdxErrorEnum.AdxError,
    )
    asset_error = proto.Field(
        proto.ENUM,
        number=107,
        oneof="error_code",
        enum=gage_asset_error.AssetErrorEnum.AssetError,
    )
    bidding_error = proto.Field(
        proto.ENUM,
        number=26,
        oneof="error_code",
        enum=gage_bidding_error.BiddingErrorEnum.BiddingError,
    )
    campaign_criterion_error = proto.Field(
        proto.ENUM,
        number=29,
        oneof="error_code",
        enum=gage_campaign_criterion_error.CampaignCriterionErrorEnum.CampaignCriterionError,
    )
    collection_size_error = proto.Field(
        proto.ENUM,
        number=31,
        oneof="error_code",
        enum=gage_collection_size_error.CollectionSizeErrorEnum.CollectionSizeError,
    )
    country_code_error = proto.Field(
        proto.ENUM,
        number=109,
        oneof="error_code",
        enum=gage_country_code_error.CountryCodeErrorEnum.CountryCodeError,
    )
    criterion_error = proto.Field(
        proto.ENUM,
        number=32,
        oneof="error_code",
        enum=gage_criterion_error.CriterionErrorEnum.CriterionError,
    )
    customer_error = proto.Field(
        proto.ENUM,
        number=90,
        oneof="error_code",
        enum=gage_customer_error.CustomerErrorEnum.CustomerError,
    )
    date_error = proto.Field(
        proto.ENUM,
        number=33,
        oneof="error_code",
        enum=gage_date_error.DateErrorEnum.DateError,
    )
    date_range_error = proto.Field(
        proto.ENUM,
        number=34,
        oneof="error_code",
        enum=gage_date_range_error.DateRangeErrorEnum.DateRangeError,
    )
    distinct_error = proto.Field(
        proto.ENUM,
        number=35,
        oneof="error_code",
        enum=gage_distinct_error.DistinctErrorEnum.DistinctError,
    )
    feed_attribute_reference_error = proto.Field(
        proto.ENUM,
        number=36,
        oneof="error_code",
        enum=gage_feed_attribute_reference_error.FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError,
    )
    function_error = proto.Field(
        proto.ENUM,
        number=37,
        oneof="error_code",
        enum=gage_function_error.FunctionErrorEnum.FunctionError,
    )
    function_parsing_error = proto.Field(
        proto.ENUM,
        number=38,
        oneof="error_code",
        enum=gage_function_parsing_error.FunctionParsingErrorEnum.FunctionParsingError,
    )
    id_error = proto.Field(
        proto.ENUM,
        number=39,
        oneof="error_code",
        enum=gage_id_error.IdErrorEnum.IdError,
    )
    image_error = proto.Field(
        proto.ENUM,
        number=40,
        oneof="error_code",
        enum=gage_image_error.ImageErrorEnum.ImageError,
    )
    language_code_error = proto.Field(
        proto.ENUM,
        number=110,
        oneof="error_code",
        enum=gage_language_code_error.LanguageCodeErrorEnum.LanguageCodeError,
    )
    media_bundle_error = proto.Field(
        proto.ENUM,
        number=42,
        oneof="error_code",
        enum=gage_media_bundle_error.MediaBundleErrorEnum.MediaBundleError,
    )
    media_upload_error = proto.Field(
        proto.ENUM,
        number=116,
        oneof="error_code",
        enum=gage_media_upload_error.MediaUploadErrorEnum.MediaUploadError,
    )
    media_file_error = proto.Field(
        proto.ENUM,
        number=86,
        oneof="error_code",
        enum=gage_media_file_error.MediaFileErrorEnum.MediaFileError,
    )
    multiplier_error = proto.Field(
        proto.ENUM,
        number=44,
        oneof="error_code",
        enum=gage_multiplier_error.MultiplierErrorEnum.MultiplierError,
    )
    new_resource_creation_error = proto.Field(
        proto.ENUM,
        number=45,
        oneof="error_code",
        enum=gage_new_resource_creation_error.NewResourceCreationErrorEnum.NewResourceCreationError,
    )
    not_empty_error = proto.Field(
        proto.ENUM,
        number=46,
        oneof="error_code",
        enum=gage_not_empty_error.NotEmptyErrorEnum.NotEmptyError,
    )
    null_error = proto.Field(
        proto.ENUM,
        number=47,
        oneof="error_code",
        enum=gage_null_error.NullErrorEnum.NullError,
    )
    operator_error = proto.Field(
        proto.ENUM,
        number=48,
        oneof="error_code",
        enum=gage_operator_error.OperatorErrorEnum.OperatorError,
    )
    range_error = proto.Field(
        proto.ENUM,
        number=49,
        oneof="error_code",
        enum=gage_range_error.RangeErrorEnum.RangeError,
    )
    recommendation_error = proto.Field(
        proto.ENUM,
        number=58,
        oneof="error_code",
        enum=gage_recommendation_error.RecommendationErrorEnum.RecommendationError,
    )
    region_code_error = proto.Field(
        proto.ENUM,
        number=51,
        oneof="error_code",
        enum=gage_region_code_error.RegionCodeErrorEnum.RegionCodeError,
    )
    setting_error = proto.Field(
        proto.ENUM,
        number=52,
        oneof="error_code",
        enum=gage_setting_error.SettingErrorEnum.SettingError,
    )
    string_format_error = proto.Field(
        proto.ENUM,
        number=53,
        oneof="error_code",
        enum=gage_string_format_error.StringFormatErrorEnum.StringFormatError,
    )
    string_length_error = proto.Field(
        proto.ENUM,
        number=54,
        oneof="error_code",
        enum=gage_string_length_error.StringLengthErrorEnum.StringLengthError,
    )
    operation_access_denied_error = proto.Field(
        proto.ENUM,
        number=55,
        oneof="error_code",
        enum=gage_operation_access_denied_error.OperationAccessDeniedErrorEnum.OperationAccessDeniedError,
    )
    resource_access_denied_error = proto.Field(
        proto.ENUM,
        number=56,
        oneof="error_code",
        enum=gage_resource_access_denied_error.ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError,
    )
    resource_count_limit_exceeded_error = proto.Field(
        proto.ENUM,
        number=57,
        oneof="error_code",
        enum=gage_resource_count_limit_exceeded_error.ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError,
    )
    youtube_video_registration_error = proto.Field(
        proto.ENUM,
        number=117,
        oneof="error_code",
        enum=gage_youtube_video_registration_error.YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError,
    )
    ad_group_bid_modifier_error = proto.Field(
        proto.ENUM,
        number=59,
        oneof="error_code",
        enum=gage_ad_group_bid_modifier_error.AdGroupBidModifierErrorEnum.AdGroupBidModifierError,
    )
    context_error = proto.Field(
        proto.ENUM,
        number=60,
        oneof="error_code",
        enum=gage_context_error.ContextErrorEnum.ContextError,
    )
    field_error = proto.Field(
        proto.ENUM,
        number=61,
        oneof="error_code",
        enum=gage_field_error.FieldErrorEnum.FieldError,
    )
    shared_set_error = proto.Field(
        proto.ENUM,
        number=62,
        oneof="error_code",
        enum=gage_shared_set_error.SharedSetErrorEnum.SharedSetError,
    )
    shared_criterion_error = proto.Field(
        proto.ENUM,
        number=63,
        oneof="error_code",
        enum=gage_shared_criterion_error.SharedCriterionErrorEnum.SharedCriterionError,
    )
    campaign_shared_set_error = proto.Field(
        proto.ENUM,
        number=64,
        oneof="error_code",
        enum=gage_campaign_shared_set_error.CampaignSharedSetErrorEnum.CampaignSharedSetError,
    )
    conversion_action_error = proto.Field(
        proto.ENUM,
        number=65,
        oneof="error_code",
        enum=gage_conversion_action_error.ConversionActionErrorEnum.ConversionActionError,
    )
    conversion_adjustment_upload_error = proto.Field(
        proto.ENUM,
        number=115,
        oneof="error_code",
        enum=gage_conversion_adjustment_upload_error.ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError,
    )
    conversion_custom_variable_error = proto.Field(
        proto.ENUM,
        number=143,
        oneof="error_code",
        enum=gage_conversion_custom_variable_error.ConversionCustomVariableErrorEnum.ConversionCustomVariableError,
    )
    conversion_upload_error = proto.Field(
        proto.ENUM,
        number=111,
        oneof="error_code",
        enum=gage_conversion_upload_error.ConversionUploadErrorEnum.ConversionUploadError,
    )
    conversion_value_rule_error = proto.Field(
        proto.ENUM,
        number=145,
        oneof="error_code",
        enum=gage_conversion_value_rule_error.ConversionValueRuleErrorEnum.ConversionValueRuleError,
    )
    conversion_value_rule_set_error = proto.Field(
        proto.ENUM,
        number=146,
        oneof="error_code",
        enum=gage_conversion_value_rule_set_error.ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError,
    )
    header_error = proto.Field(
        proto.ENUM,
        number=66,
        oneof="error_code",
        enum=gage_header_error.HeaderErrorEnum.HeaderError,
    )
    database_error = proto.Field(
        proto.ENUM,
        number=67,
        oneof="error_code",
        enum=gage_database_error.DatabaseErrorEnum.DatabaseError,
    )
    policy_finding_error = proto.Field(
        proto.ENUM,
        number=68,
        oneof="error_code",
        enum=gage_policy_finding_error.PolicyFindingErrorEnum.PolicyFindingError,
    )
    enum_error = proto.Field(
        proto.ENUM,
        number=70,
        oneof="error_code",
        enum=gage_enum_error.EnumErrorEnum.EnumError,
    )
    keyword_plan_error = proto.Field(
        proto.ENUM,
        number=71,
        oneof="error_code",
        enum=gage_keyword_plan_error.KeywordPlanErrorEnum.KeywordPlanError,
    )
    keyword_plan_campaign_error = proto.Field(
        proto.ENUM,
        number=72,
        oneof="error_code",
        enum=gage_keyword_plan_campaign_error.KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError,
    )
    keyword_plan_campaign_keyword_error = proto.Field(
        proto.ENUM,
        number=132,
        oneof="error_code",
        enum=gage_keyword_plan_campaign_keyword_error.KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError,
    )
    keyword_plan_ad_group_error = proto.Field(
        proto.ENUM,
        number=74,
        oneof="error_code",
        enum=gage_keyword_plan_ad_group_error.KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError,
    )
    keyword_plan_ad_group_keyword_error = proto.Field(
        proto.ENUM,
        number=133,
        oneof="error_code",
        enum=gage_keyword_plan_ad_group_keyword_error.KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError,
    )
    keyword_plan_idea_error = proto.Field(
        proto.ENUM,
        number=76,
        oneof="error_code",
        enum=gage_keyword_plan_idea_error.KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError,
    )
    account_budget_proposal_error = proto.Field(
        proto.ENUM,
        number=77,
        oneof="error_code",
        enum=gage_account_budget_proposal_error.AccountBudgetProposalErrorEnum.AccountBudgetProposalError,
    )
    user_list_error = proto.Field(
        proto.ENUM,
        number=78,
        oneof="error_code",
        enum=gage_user_list_error.UserListErrorEnum.UserListError,
    )
    change_event_error = proto.Field(
        proto.ENUM,
        number=136,
        oneof="error_code",
        enum=gage_change_event_error.ChangeEventErrorEnum.ChangeEventError,
    )
    change_status_error = proto.Field(
        proto.ENUM,
        number=79,
        oneof="error_code",
        enum=gage_change_status_error.ChangeStatusErrorEnum.ChangeStatusError,
    )
    feed_error = proto.Field(
        proto.ENUM,
        number=80,
        oneof="error_code",
        enum=gage_feed_error.FeedErrorEnum.FeedError,
    )
    geo_target_constant_suggestion_error = proto.Field(
        proto.ENUM,
        number=81,
        oneof="error_code",
        enum=gage_geo_target_constant_suggestion_error.GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError,
    )
    campaign_draft_error = proto.Field(
        proto.ENUM,
        number=82,
        oneof="error_code",
        enum=gage_campaign_draft_error.CampaignDraftErrorEnum.CampaignDraftError,
    )
    feed_item_error = proto.Field(
        proto.ENUM,
        number=83,
        oneof="error_code",
        enum=gage_feed_item_error.FeedItemErrorEnum.FeedItemError,
    )
    label_error = proto.Field(
        proto.ENUM,
        number=84,
        oneof="error_code",
        enum=gage_label_error.LabelErrorEnum.LabelError,
    )
    billing_setup_error = proto.Field(
        proto.ENUM,
        number=87,
        oneof="error_code",
        enum=gage_billing_setup_error.BillingSetupErrorEnum.BillingSetupError,
    )
    customer_client_link_error = proto.Field(
        proto.ENUM,
        number=88,
        oneof="error_code",
        enum=gage_customer_client_link_error.CustomerClientLinkErrorEnum.CustomerClientLinkError,
    )
    customer_manager_link_error = proto.Field(
        proto.ENUM,
        number=91,
        oneof="error_code",
        enum=gage_customer_manager_link_error.CustomerManagerLinkErrorEnum.CustomerManagerLinkError,
    )
    feed_mapping_error = proto.Field(
        proto.ENUM,
        number=92,
        oneof="error_code",
        enum=gage_feed_mapping_error.FeedMappingErrorEnum.FeedMappingError,
    )
    customer_feed_error = proto.Field(
        proto.ENUM,
        number=93,
        oneof="error_code",
        enum=gage_customer_feed_error.CustomerFeedErrorEnum.CustomerFeedError,
    )
    ad_group_feed_error = proto.Field(
        proto.ENUM,
        number=94,
        oneof="error_code",
        enum=gage_ad_group_feed_error.AdGroupFeedErrorEnum.AdGroupFeedError,
    )
    campaign_feed_error = proto.Field(
        proto.ENUM,
        number=96,
        oneof="error_code",
        enum=gage_campaign_feed_error.CampaignFeedErrorEnum.CampaignFeedError,
    )
    custom_interest_error = proto.Field(
        proto.ENUM,
        number=97,
        oneof="error_code",
        enum=gage_custom_interest_error.CustomInterestErrorEnum.CustomInterestError,
    )
    campaign_experiment_error = proto.Field(
        proto.ENUM,
        number=98,
        oneof="error_code",
        enum=gage_campaign_experiment_error.CampaignExperimentErrorEnum.CampaignExperimentError,
    )
    extension_feed_item_error = proto.Field(
        proto.ENUM,
        number=100,
        oneof="error_code",
        enum=gage_extension_feed_item_error.ExtensionFeedItemErrorEnum.ExtensionFeedItemError,
    )
    ad_parameter_error = proto.Field(
        proto.ENUM,
        number=101,
        oneof="error_code",
        enum=gage_ad_parameter_error.AdParameterErrorEnum.AdParameterError,
    )
    feed_item_validation_error = proto.Field(
        proto.ENUM,
        number=102,
        oneof="error_code",
        enum=gage_feed_item_validation_error.FeedItemValidationErrorEnum.FeedItemValidationError,
    )
    extension_setting_error = proto.Field(
        proto.ENUM,
        number=103,
        oneof="error_code",
        enum=gage_extension_setting_error.ExtensionSettingErrorEnum.ExtensionSettingError,
    )
    feed_item_set_error = proto.Field(
        proto.ENUM,
        number=140,
        oneof="error_code",
        enum=gage_feed_item_set_error.FeedItemSetErrorEnum.FeedItemSetError,
    )
    feed_item_set_link_error = proto.Field(
        proto.ENUM,
        number=141,
        oneof="error_code",
        enum=gage_feed_item_set_link_error.FeedItemSetLinkErrorEnum.FeedItemSetLinkError,
    )
    feed_item_target_error = proto.Field(
        proto.ENUM,
        number=104,
        oneof="error_code",
        enum=gage_feed_item_target_error.FeedItemTargetErrorEnum.FeedItemTargetError,
    )
    policy_violation_error = proto.Field(
        proto.ENUM,
        number=105,
        oneof="error_code",
        enum=gage_policy_violation_error.PolicyViolationErrorEnum.PolicyViolationError,
    )
    partial_failure_error = proto.Field(
        proto.ENUM,
        number=112,
        oneof="error_code",
        enum=gage_partial_failure_error.PartialFailureErrorEnum.PartialFailureError,
    )
    policy_validation_parameter_error = proto.Field(
        proto.ENUM,
        number=114,
        oneof="error_code",
        enum=gage_policy_validation_parameter_error.PolicyValidationParameterErrorEnum.PolicyValidationParameterError,
    )
    size_limit_error = proto.Field(
        proto.ENUM,
        number=118,
        oneof="error_code",
        enum=gage_size_limit_error.SizeLimitErrorEnum.SizeLimitError,
    )
    offline_user_data_job_error = proto.Field(
        proto.ENUM,
        number=119,
        oneof="error_code",
        enum=gage_offline_user_data_job_error.OfflineUserDataJobErrorEnum.OfflineUserDataJobError,
    )
    not_allowlisted_error = proto.Field(
        proto.ENUM,
        number=137,
        oneof="error_code",
        enum=gage_not_allowlisted_error.NotAllowlistedErrorEnum.NotAllowlistedError,
    )
    manager_link_error = proto.Field(
        proto.ENUM,
        number=121,
        oneof="error_code",
        enum=gage_manager_link_error.ManagerLinkErrorEnum.ManagerLinkError,
    )
    currency_code_error = proto.Field(
        proto.ENUM,
        number=122,
        oneof="error_code",
        enum=gage_currency_code_error.CurrencyCodeErrorEnum.CurrencyCodeError,
    )
    access_invitation_error = proto.Field(
        proto.ENUM,
        number=124,
        oneof="error_code",
        enum=gage_access_invitation_error.AccessInvitationErrorEnum.AccessInvitationError,
    )
    reach_plan_error = proto.Field(
        proto.ENUM,
        number=125,
        oneof="error_code",
        enum=gage_reach_plan_error.ReachPlanErrorEnum.ReachPlanError,
    )
    invoice_error = proto.Field(
        proto.ENUM,
        number=126,
        oneof="error_code",
        enum=gage_invoice_error.InvoiceErrorEnum.InvoiceError,
    )
    payments_account_error = proto.Field(
        proto.ENUM,
        number=127,
        oneof="error_code",
        enum=gage_payments_account_error.PaymentsAccountErrorEnum.PaymentsAccountError,
    )
    time_zone_error = proto.Field(
        proto.ENUM,
        number=128,
        oneof="error_code",
        enum=gage_time_zone_error.TimeZoneErrorEnum.TimeZoneError,
    )
    asset_link_error = proto.Field(
        proto.ENUM,
        number=129,
        oneof="error_code",
        enum=gage_asset_link_error.AssetLinkErrorEnum.AssetLinkError,
    )
    user_data_error = proto.Field(
        proto.ENUM,
        number=130,
        oneof="error_code",
        enum=gage_user_data_error.UserDataErrorEnum.UserDataError,
    )
    batch_job_error = proto.Field(
        proto.ENUM,
        number=131,
        oneof="error_code",
        enum=gage_batch_job_error.BatchJobErrorEnum.BatchJobError,
    )
    account_link_error = proto.Field(
        proto.ENUM,
        number=134,
        oneof="error_code",
        enum=gage_account_link_error.AccountLinkErrorEnum.AccountLinkError,
    )
    third_party_app_analytics_link_error = proto.Field(
        proto.ENUM,
        number=135,
        oneof="error_code",
        enum=gage_third_party_app_analytics_link_error.ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError,
    )
    customer_user_access_error = proto.Field(
        proto.ENUM,
        number=138,
        oneof="error_code",
        enum=gage_customer_user_access_error.CustomerUserAccessErrorEnum.CustomerUserAccessError,
    )
    custom_audience_error = proto.Field(
        proto.ENUM,
        number=139,
        oneof="error_code",
        enum=gage_custom_audience_error.CustomAudienceErrorEnum.CustomAudienceError,
    )


class ErrorLocation(proto.Message):
    r"""Describes the part of the request proto that caused the
    error.

    Attributes:
        field_path_elements (Sequence[google.ads.googleads.v8.errors.types.ErrorLocation.FieldPathElement]):
            A field path that indicates which field was
            invalid in the request.
    """

    class FieldPathElement(proto.Message):
        r"""A part of a field path.
        Attributes:
            field_name (str):
                The name of a field or a oneof
            index (int):
                If field_name is a repeated field, this is the element that
                failed
        """

        field_name = proto.Field(proto.STRING, number=1,)
        index = proto.Field(proto.INT32, number=3, optional=True,)

    field_path_elements = proto.RepeatedField(
        proto.MESSAGE, number=2, message=FieldPathElement,
    )


class ErrorDetails(proto.Message):
    r"""Additional error details.
    Attributes:
        unpublished_error_code (str):
            The error code that should have been
            returned, but wasn't. This is used when the
            error code is not published in the client
            specified version.
        policy_violation_details (google.ads.googleads.v8.errors.types.PolicyViolationDetails):
            Describes an ad policy violation.
        policy_finding_details (google.ads.googleads.v8.errors.types.PolicyFindingDetails):
            Describes policy violation findings.
        quota_error_details (google.ads.googleads.v8.errors.types.QuotaErrorDetails):
            Details on the quota error, including the
            scope (account or developer), the rate bucket
            name and the retry delay.
        resource_count_details (google.ads.googleads.v8.errors.types.ResourceCountDetails):
            Details for a resource count limit exceeded
            error.
    """

    unpublished_error_code = proto.Field(proto.STRING, number=1,)
    policy_violation_details = proto.Field(
        proto.MESSAGE, number=2, message="PolicyViolationDetails",
    )
    policy_finding_details = proto.Field(
        proto.MESSAGE, number=3, message="PolicyFindingDetails",
    )
    quota_error_details = proto.Field(
        proto.MESSAGE, number=4, message="QuotaErrorDetails",
    )
    resource_count_details = proto.Field(
        proto.MESSAGE, number=5, message="ResourceCountDetails",
    )


class PolicyViolationDetails(proto.Message):
    r"""Error returned as part of a mutate response.
    This error indicates single policy violation by some text in one
    of the fields.

    Attributes:
        external_policy_description (str):
            Human readable description of policy
            violation.
        key (google.ads.googleads.v8.common.types.PolicyViolationKey):
            Unique identifier for this violation.
            If policy is exemptible, this key may be used to
            request exemption.
        external_policy_name (str):
            Human readable name of the policy.
        is_exemptible (bool):
            Whether user can file an exemption request
            for this violation.
    """

    external_policy_description = proto.Field(proto.STRING, number=2,)
    key = proto.Field(
        proto.MESSAGE, number=4, message=policy.PolicyViolationKey,
    )
    external_policy_name = proto.Field(proto.STRING, number=5,)
    is_exemptible = proto.Field(proto.BOOL, number=6,)


class PolicyFindingDetails(proto.Message):
    r"""Error returned as part of a mutate response.
    This error indicates one or more policy findings in the fields
    of a resource.

    Attributes:
        policy_topic_entries (Sequence[google.ads.googleads.v8.common.types.PolicyTopicEntry]):
            The list of policy topics for the resource. Contains the
            PROHIBITED or FULLY_LIMITED policy topic entries that
            prevented the resource from being saved (among any other
            entries the resource may also have).
    """

    policy_topic_entries = proto.RepeatedField(
        proto.MESSAGE, number=1, message=policy.PolicyTopicEntry,
    )


class QuotaErrorDetails(proto.Message):
    r"""Additional quota error details when there is QuotaError.
    Attributes:
        rate_scope (google.ads.googleads.v8.errors.types.QuotaErrorDetails.QuotaRateScope):
            The rate scope of the quota limit.
        rate_name (str):
            The high level description of the quota
            bucket. Examples are "Get requests for standard
            access" or "Requests per account".
        retry_delay (google.protobuf.duration_pb2.Duration):
            Backoff period that customers should wait
            before sending next request.
    """

    class QuotaRateScope(proto.Enum):
        r"""Enum of possible scopes that quota buckets belong to."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNT = 2
        DEVELOPER = 3

    rate_scope = proto.Field(proto.ENUM, number=1, enum=QuotaRateScope,)
    rate_name = proto.Field(proto.STRING, number=2,)
    retry_delay = proto.Field(
        proto.MESSAGE, number=3, message=duration_pb2.Duration,
    )


class ResourceCountDetails(proto.Message):
    r"""Error details returned when an resource count limit was
    exceeded.

    Attributes:
        enclosing_id (str):
            The ID of the resource whose limit was
            exceeded. External customer ID if the limit is
            for a customer.
        enclosing_resource (str):
            The name of the resource (Customer, Campaign
            etc.) whose limit was exceeded.
        limit (int):
            The limit which was exceeded.
        limit_type (google.ads.googleads.v8.enums.types.ResourceLimitTypeEnum.ResourceLimitType):
            The resource limit type which was exceeded.
        existing_count (int):
            The count of existing entities.
    """

    enclosing_id = proto.Field(proto.STRING, number=1,)
    enclosing_resource = proto.Field(proto.STRING, number=5,)
    limit = proto.Field(proto.INT32, number=2,)
    limit_type = proto.Field(
        proto.ENUM,
        number=3,
        enum=resource_limit_type.ResourceLimitTypeEnum.ResourceLimitType,
    )
    existing_count = proto.Field(proto.INT32, number=4,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.common.types import criteria
from google.ads.googleads.v8.common.types import extensions
from google.ads.googleads.v8.enums.types import (
    extension_type as gage_extension_type,
)
from google.ads.googleads.v8.enums.types import feed_item_status
from google.ads.googleads.v8.enums.types import feed_item_target_device


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.resources",
    marshal="google.ads.googleads.v8",
    manifest={"ExtensionFeedItem",},
)


class ExtensionFeedItem(proto.Message):
    r"""An extension feed item.
    Attributes:
        resource_name (str):
            Immutable. The resource name of the extension feed item.
            Extension feed item resource names have the form:

            ``customers/{customer_id}/extensionFeedItems/{feed_item_id}``
        id (int):
            Output only. The ID of this feed item. Read-
            nly.
        extension_type (google.ads.googleads.v8.enums.types.ExtensionTypeEnum.ExtensionType):
            Output only. The extension type of the
            extension feed item. This field is read-only.
        start_date_time (str):
            Start time in which this feed item is
            effective and can begin serving. The time is in
            the customer's time zone. The format is "YYYY-
            MM-DD HH:MM:SS".
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".
        end_date_time (str):
            End time in which this feed item is no longer
            effective and will stop serving. The time is in
            the customer's time zone. The format is "YYYY-
            MM-DD HH:MM:SS".
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".
        ad_schedules (Sequence[google.ads.googleads.v8.common.types.AdScheduleInfo]):
            List of non-overlapping schedules specifying
            all time intervals for which the feed item may
            serve. There can be a maximum of 6 schedules per
            day.
        device (google.ads.googleads.v8.enums.types.FeedItemTargetDeviceEnum.FeedItemTargetDevice):
            The targeted device.
        targeted_geo_target_constant (str):
            The targeted geo target constant.
        targeted_keyword (google.ads.googleads.v8.common.types.KeywordInfo):
            The targeted keyword.
        status (google.ads.googleads.v8.enums.types.FeedItemStatusEnum.FeedItemStatus):
            Output only. Status of the feed item.
            This field is read-only.
        sitelink_feed_item (google.ads.googleads.v8.common.types.SitelinkFeedItem):
            Sitelink extension.
        structured_snippet_feed_item (google.ads.googleads.v8.common.types.StructuredSnippetFeedItem):
            Structured snippet extension.
        app_feed_item (google.ads.googleads.v8.common.types.AppFeedItem):
            App extension.
        call_feed_item (google.ads.googleads.v8.common.types.CallFeedItem):
            Call extension.
        callout_feed_item (google.ads.googleads.v8.common.types.CalloutFeedItem):
            Callout extension.
        text_message_feed_item (google.ads.googleads.v8.common.types.TextMessageFeedItem):
            Text message extension.
        price_feed_item (google.ads.googleads.v8.common.types.PriceFeedItem):
            Price extension.
        promotion_feed_item (google.ads.googleads.v8.common.types.PromotionFeedItem):
            Promotion extension.
        location_feed_item (google.ads.googleads.v8.common.types.LocationFeedItem):
            Output only. Location extension. Locations
            are synced from a GMB account into a feed. This
            field is read-only.
        affiliate_location_feed_item (google.ads.googleads.v8.common.types.AffiliateLocationFeedItem):
            Output only. Affiliate location extension.
            Feed locations are populated by Google Ads based
            on a chain ID. This field is read-only.
        hotel_callout_feed_item (google.ads.googleads.v8.common.types.HotelCalloutFeedItem):
            Hotel Callout extension.
        image_feed_item (google.ads.googleads.v8.common.types.ImageFeedItem):
            Immutable. Advertiser provided image
            extension.
        targeted_campaign (str):
            The targeted campaign.
        targeted_ad_group (str):
            The targeted ad group.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.INT64, number=25, optional=True,)
    extension_type = proto.Field(
        proto.ENUM,
        number=13,
        enum=gage_extension_type.ExtensionTypeEnum.ExtensionType,
    )
    start_date_time = proto.Field(proto.STRING, number=26, optional=True,)
    end_date_time = proto.Field(proto.STRING, number=27, optional=True,)
    ad_schedules = proto.RepeatedField(
        proto.MESSAGE, number=16, message=criteria.AdScheduleInfo,
    )
    device = proto.Field(
        proto.ENUM,
        number=17,
        enum=feed_item_target_device.FeedItemTargetDeviceEnum.FeedItemTargetDevice,
    )
    targeted_geo_target_constant = proto.Field(
        proto.STRING, number=30, optional=True,
    )
    targeted_keyword = proto.Field(
        proto.MESSAGE, number=22, message=criteria.KeywordInfo,
    )
    status = proto.Field(
        proto.ENUM,
        number=4,
        enum=feed_item_status.FeedItemStatusEnum.FeedItemStatus,
    )
    sitelink_feed_item = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="extension",
        message=extensions.SitelinkFeedItem,
    )
    structured_snippet_feed_item = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="extension",
        message=extensions.StructuredSnippetFeedItem,
    )
    app_feed_item = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="extension",
        message=extensions.AppFeedItem,
    )
    call_feed_item = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="extension",
        message=extensions.CallFeedItem,
    )
    callout_feed_item = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="extension",
        message=extensions.CalloutFeedItem,
    )
    text_message_feed_item = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="extension",
        message=extensions.TextMessageFeedItem,
    )
    price_feed_item = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="extension",
        message=extensions.PriceFeedItem,
    )
    promotion_feed_item = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="extension",
        message=extensions.PromotionFeedItem,
    )
    location_feed_item = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="extension",
        message=extensions.LocationFeedItem,
    )
    affiliate_location_feed_item = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="extension",
        message=extensions.AffiliateLocationFeedItem,
    )
    hotel_callout_feed_item = proto.Field(
        proto.MESSAGE,
        number=23,
        oneof="extension",
        message=extensions.HotelCalloutFeedItem,
    )
    image_feed_item = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="extension",
        message=extensions.ImageFeedItem,
    )
    targeted_campaign = proto.Field(
        proto.STRING, number=28, oneof="serving_resource_targeting",
    )
    targeted_ad_group = proto.Field(
        proto.STRING, number=29, oneof="serving_resource_targeting",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A gRPC Interceptor that is responsible to augmenting request metadata.

This class is initialized in the GoogleAdsClient and passed into a grpc
intercept_channel whenever a new service is initialized. It intercepts requests
and updates the metadata in order to insert the developer token and
login-customer-id values.
"""

from grpc import UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor

from .interceptor import Interceptor


class MetadataInterceptor(
    Interceptor, UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor
):
    """An interceptor that appends custom metadata to requests."""

    def __init__(
        self, developer_token, login_customer_id, linked_customer_id=None
    ):
        """Initialization method for this class.

        Args:
            developer_token: a str developer token.
            login_customer_id: a str specifying a login customer ID.
            linked_customer_id: a str specifying a linked customer ID.
        """
        self.developer_token_meta = ("developer-token", developer_token)
        self.login_customer_id_meta = (
            ("login-customer-id", login_customer_id)
            if login_customer_id
            else None
        )
        self.linked_customer_id_meta = (
            ("linked-customer-id", linked_customer_id)
            if linked_customer_id
            else None
        )

    def _update_client_call_details_metadata(
        self, client_call_details, metadata
    ):
        """Updates the client call details with additional metadata.

        Args:
            client_call_details: An instance of grpc.ClientCallDetails.
            metadata: Additional metadata defined by GoogleAdsClient.

        Returns:
            An new instance of grpc.ClientCallDetails with additional metadata
            from the GoogleAdsClient.
        """
        client_call_details = self.get_client_call_details_instance(
            client_call_details.method,
            client_call_details.timeout,
            metadata,
            client_call_details.credentials,
        )

        return client_call_details

    def _intercept(self, continuation, client_call_details, request):
        """Generic interceptor used for Unary-Unary and Unary-Stream requests.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        if client_call_details.metadata is None:
            metadata = []
        else:
            metadata = list(client_call_details.metadata)

        metadata.append(self.developer_token_meta)

        if self.login_customer_id_meta:
            metadata.append(self.login_customer_id_meta)

        if self.linked_customer_id_meta:
            metadata.append(self.linked_customer_id_meta)

        client_call_details = self._update_client_call_details_metadata(
            client_call_details, metadata
        )

        return continuation(client_call_details, request)

    def intercept_unary_unary(self, continuation, client_call_details, request):
        """Intercepts and appends custom metadata for Unary-Unary requests.

        Overrides abstract method defined in grpc.UnaryUnaryClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        return self._intercept(continuation, client_call_details, request)

    def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        """Intercepts and appends custom metadata to Unary-Stream requests.

        Overrides abstract method defined in grpc.UnaryStreamClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        return self._intercept(continuation, client_call_details, request)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v8.resources.types import (
    customer_user_access_invitation,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v8.services",
    marshal="google.ads.googleads.v8",
    manifest={
        "GetCustomerUserAccessInvitationRequest",
        "MutateCustomerUserAccessInvitationRequest",
        "CustomerUserAccessInvitationOperation",
        "MutateCustomerUserAccessInvitationResponse",
        "MutateCustomerUserAccessInvitationResult",
    },
)


class GetCustomerUserAccessInvitationRequest(proto.Message):
    r"""Request message for
    [CustomerUserAccessInvitation.GetCustomerUserAccessInvitation][]

    Attributes:
        resource_name (str):
            Required. Resource name of the access
            invitation.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class MutateCustomerUserAccessInvitationRequest(proto.Message):
    r"""Request message for
    [CustomerUserAccessInvitation.MutateCustomerUserAccessInvitation][]

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose access
            invitation is being modified.
        operation (google.ads.googleads.v8.services.types.CustomerUserAccessInvitationOperation):
            Required. The operation to perform on the
            access invitation
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operation = proto.Field(
        proto.MESSAGE,
        number=2,
        message="CustomerUserAccessInvitationOperation",
    )


class CustomerUserAccessInvitationOperation(proto.Message):
    r"""A single operation (create or remove) on customer user access
    invitation.

    Attributes:
        create (google.ads.googleads.v8.resources.types.CustomerUserAccessInvitation):
            Create operation: No resource name is
            expected for the new access invitation.
        remove (str):
            Remove operation: A resource name for the revoke invitation
            is expected, in this format:

            ``customers/{customer_id}/customerUserAccessInvitations/{invitation_id}``
    """

    create = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=customer_user_access_invitation.CustomerUserAccessInvitation,
    )
    remove = proto.Field(proto.STRING, number=2, oneof="operation",)


class MutateCustomerUserAccessInvitationResponse(proto.Message):
    r"""Response message for access invitation mutate.
    Attributes:
        result (google.ads.googleads.v8.services.types.MutateCustomerUserAccessInvitationResult):
            Result for the mutate.
    """

    result = proto.Field(
        proto.MESSAGE,
        number=1,
        message="MutateCustomerUserAccessInvitationResult",
    )


class MutateCustomerUserAccessInvitationResult(proto.Message):
    r"""The result for the access invitation mutate.
    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))
